/*
 * XmlDocumentIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Authors: Ghostar, TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Globalization;
using System.IO;
using System.Text;
using System.Xml;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.Xml
{
    /// <summary>
    /// Implements the <see cref="IDocumentIndexSource"/> which provides document index persistence
    /// in an Xml file.
    /// </summary>
    ///
    /// <remarks>
    /// It saves to and loads from one or more Xml files, depending on a limit defined by the user.
    /// Besides just the standard saving to an Xml file, this class also provides the option
    /// to split the saving of the document index to multiple XML files, all linked from the main file.
    /// This provides a way to get around file system size limits on a single file.
    /// Note that <see cref="GetEnumerator"/> returns an <see cref="IDictionaryEnumerator"/>
    /// which is not thread-safe, but a <see cref="SyncRoot"/> is provided.
    /// </remarks>
    ///
    /// <example>
    /// Here's a sample single file xml: <code><![CDATA[
    /// <?xml version="1.0" encoding="utf-8"?>
    /// <Document>
    ///     <CompareOptions>None</CompareOptions>
    ///     <Culture>en-US</Culture>
    ///     <Words>
    ///         <WordsList>
    ///             <Word>2005</Word>
    ///             <Locations>
    ///                 <Location>2</Location>
    ///             </Locations>
    ///             <Word>Open</Word>
    ///             <Locations>
    ///                 <Location>1</Location>
    ///                 <Location>9</Location>
    ///             </Locations>
    ///         </WordsList>
    ///     </Words>
    /// </Document>
    /// ]]></code>
    /// Here's a sample master file with its sub file: <code><![CDATA[
    /// <?xml version="1.0" encoding="utf-8"?>
    /// <Document>
    ///     <CompareOptions>None</CompareOptions>
    ///     <Culture>en-US</Culture>
    ///     <Words>
    ///         <Files>
    ///             <FilePath>..\..\test_files\Document2(883bba12-afad-42f7-9780-86359f285cad).xml</FilePath>
    ///             <FilePath>..\..\test_files\Document2(b3aed422-4354-46ad-b88d-d0f07285054b).xml</FilePath>
    ///         </Files>
    ///     </Words>
    /// </Document>
    ///
    /// <?xml version="1.0" encoding="utf-8"?>
    /// <WordsList>
    ///     <Word>Persistence</Word>
    ///     <Locations>
    ///         <Location>5</Location>
    ///     </Locations>
    ///     <Word>TopCoder</Word>
    ///     <Locations>
    ///         <Location>6</Location>
    ///         <Location>8</Location>
    ///     </Locations>
    /// </WordsList>
    /// ]]></code>
    /// Here's a sample code: <code><![CDATA[
    /// // load document words' indices
    /// IDocumentIndexSource source = new XmlDocumentIndexSource();
    /// IParameters documentParams = new SimpleIndexParameters(docFile);
    /// source.Load(documentParams);
    ///
    /// // check CompareOptions and Culture
    /// CompareOptions opt = source.CompareOptions;
    /// CultureInfo ci = source.Culture;
    ///
    /// // check IndexInfo
    /// IndexInfo idxInfo = source.IndexInfo;
    ///
    /// // enumerate the word indices
    /// IDictionaryEnumerator enumerator = source.GetEnumerator();
    /// int count = 0;
    /// while (enumerator.MoveNext())
    /// {
    ///     count++;
    /// }
    ///
    /// // add a new word
    /// WordInfo word = new WordInfo("Cat", 102);
    /// source.Add(word);
    ///
    /// // add using indexer
    /// IList pos = new ArrayList();
    /// pos.Add(103);
    /// pos.Add(105);
    /// source["Dog"] = pos;
    ///
    /// // save changes
    /// source.Save();
    ///
    /// // check word exists
    /// bool exists = source.Contains("Cat");
    ///
    /// // remove a word
    /// source.Remove("Cat");
    ///
    /// // use indexer
    /// IList getPos = source["Dog"];
    /// int ct = getPos.Count;
    ///
    /// // set a new file path
    /// ((XmlDocumentIndexSource) source).FilePath = subFiles;
    ///
    /// // save to multiple files
    /// ((XmlDocumentIndexSource) source).NumberOfSubFiles = 1;
    /// source.Save();
    ///
    /// // clear all words
    /// source.Clear();
    ///
    /// // save the empty file
    /// source.Save();
    /// ]]></code>
    /// </example>
    ///
    /// <threadsafety static="true" instance="true"/>
    public class XmlDocumentIndexSource : IDocumentIndexSource, IComparer
    {
        /// <summary>
        /// A common Xml Schema for a WordsList document type xml file.
        /// </summary>
        private const string WordsListXmlSchema =
            @"<xs:complexType name=""WordsList"">"
                + @"<xs:sequence minOccurs=""0"" maxOccurs=""unbounded"">"
                    + @"<xs:element name=""Word"" type=""xs:string""/>"
                    + @"<xs:element name=""Locations"" type=""Locations""/>"
                + @"</xs:sequence>"
            + @"</xs:complexType>"
            + @"<xs:complexType name=""Locations"">"
                + @"<xs:sequence maxOccurs=""unbounded"">"
                    + @"<xs:element name=""Location"" type=""xs:integer""/>"
                + @"</xs:sequence>"
            + @"</xs:complexType>";

        /// <summary>
        /// The Xml Schema for a word xml file.
        /// Used to validate the loaded xml.
        /// </summary>
        private const string WordsXmlSchema =
            @"<?xml version=""1.0"" encoding=""UTF-8""?>"
            + @"<xs:schema xmlns:xs=""http://www.w3.org/2001/XMLSchema"">"
                + @"<xs:element name=""WordsList"" type=""WordsList""/>"
                + WordsListXmlSchema
            + @"</xs:schema>";

        /// <summary>
        /// The Xml Schema for a document xml file.
        /// Used to validate the loaded xml.
        /// </summary>
        private const string DocumentXmlSchema =
            @"<?xml version=""1.0"" encoding=""UTF-8""?>"
            + @"<xs:schema xmlns:xs=""http://www.w3.org/2001/XMLSchema"">"
                + @"<xs:element name=""Document"" type=""Document""/>"
                + @"<xs:complexType name=""Document"">"
                    + @"<xs:sequence>"
                        + @"<xs:element name=""CompareOptions"" type=""xs:string""/>"
                        + @"<xs:element name=""Culture"" type=""xs:string""/>"
                        + @"<xs:element name=""Words"" type=""Words""/>"
                    + @"</xs:sequence>"
                + @"</xs:complexType>"

                + @"<xs:complexType name=""Words"">"
                    + @"<xs:sequence>"
                        + @"<xs:element name=""WordsList"" type=""WordsList"" minOccurs=""0""/>"
                        + @"<xs:element name=""Files"" type=""Files"" minOccurs=""0""/>"
                    + @"</xs:sequence>"
                + @"</xs:complexType>"

                + @"<xs:complexType name=""Files"">"
                    + @"<xs:sequence maxOccurs=""unbounded"">"
                        + @"<xs:element name=""FilePath"" type=""xs:string""/>"
                    + @"</xs:sequence>"
                + @"</xs:complexType>"

                + WordsListXmlSchema

            + @"</xs:schema>";

        /// <summary>
        /// Xml element tag for root document.
        /// </summary>
        private const string DocumentElement = "Document";

        /// <summary>
        /// Xml element tag for CompareOptions.
        /// </summary>
        private const string CompareOptionsElement = "CompareOptions";

        /// <summary>
        /// Xml element tag for Culture.
        /// </summary>
        private const string CultureElement = "Culture";

        /// <summary>
        /// Xml element tag for Words.
        /// </summary>
        private const string WordsElement = "Words";

        /// <summary>
        /// Xml element tag for WordsList.
        /// </summary>
        private const string WordsListElement = "WordsList";

        /// <summary>
        /// Xml element tag for Files.
        /// </summary>
        private const string FilesElement = "Files";

        /// <summary>
        /// Xml element tag for FilePath.
        /// </summary>
        private const string FilePathElement = "FilePath";

        /// <summary>
        /// Xml element tag for Word.
        /// </summary>
        private const string WordElement = "Word";

        /// <summary>
        /// Xml element tag for Locations.
        /// </summary>
        private const string LocationsElement = "Locations";

        /// <summary>
        /// Xml element tag for Location.
        /// </summary>
        private const string LocationElement = "Location";

        /// <summary>
        /// XPath to select all SubFiles: /Document/Words/Files/FilePath.
        /// </summary>
        private const string XPathSelectSubFiles =
            "/" + DocumentElement + "/" + WordsElement + "/" + FilesElement + "/" + FilePathElement;

        /// <summary>
        /// XPath to select WordsList: /Document/Words/WordsList.
        /// </summary>
        private const string XPathSelectWordsList =
            "/" + DocumentElement + "/" + WordsElement + "/" + WordsListElement;

        /// <summary>
        /// XPath to select CompareOptions: /Document/CompareOptions.
        /// </summary>
        private const string XPathSelectCompareOptions =
            "/" + DocumentElement + "/" + CompareOptionsElement;

        /// <summary>
        /// XPath to select WordsList: /Document/Culture.
        /// </summary>
        private const string XPathSelectCulture =
            "/" + DocumentElement + "/" + CultureElement;

        /// <summary>
        /// A unique sub file name to generate.
        /// {0} is the path + original file name (without extension), {1} is the unique GUID.
        /// </summary>
        private const string UniqueSubFileName = @"{0}({1}).xml";

        /// <summary>
        /// This private member variables holds the culture to use when comparing two strings.
        /// This allows us to provide support for multiple languages and alphabets
        /// when searching through a given document.
        /// This member variable is initialized to a default value, and
        /// it can be changed through the Culture property.
        /// </summary>
        private CultureInfo culture = new CultureInfo(Helper.DefaultCulture);

        /// <summary>
        /// This member variable holds options, beyond the culture, for comparing strings
        /// in this particular class. This  value is initialized to a default, and
        /// it can be accessed using the CompareOptions property.
        /// </summary>
        private CompareOptions compareOptions = Helper.DefaultCompareOptions;

        /// <summary>
        /// This member variable holds the file path to where the index should be saved.
        /// This member variable is set in the constructor, as well as using the FilePath property.
        /// It is accessed by the Save() method when saving to a particular file.
        /// </summary>
        private string filePath;

        /// <summary>
        /// This member variable holds the number of sub files to split the words across
        /// when saving them to the file system. This allows the user to get as fine grained as they want,
        /// in terms of individual file size.
        /// This value defaults to 0, in which case all the words will be in the original file.
        /// This value can be retrieved and set using the NumberOfSubFiles property.
        /// </summary>
        private int numberOfSubFiles = 0;

        /// <summary>
        /// This member variable holds the loaded words, loaded from the XML file,
        /// that this instance is working against.
        /// The keys in this dictionary are the words, and the values are ILists of long locations
        /// in the document where this word can be found.
        /// This collection is loaded by the Load method and accessed by the indexers in this class.
        /// </summary>
        private IDictionary words;

        /// <summary>
        /// Gets the object to use when synchronizing access to the indices collection.
        /// </summary>
        ///
        /// <value>
        /// The object to use when synchronizing access to the indices collection.
        /// </value>
        public object SyncRoot
        {
            get
            {
                return words.SyncRoot;
            }
        }

        /// <summary>
        /// Gets a newly created <see cref="IndexInfo"/> instance, created with the assembly
        /// and file name of this class, along with a <see cref="SimpleIndexParameters"/>
        /// instance that holds the path to the file that the current instance is currently
        /// working against.
        /// </summary>
        ///
        /// <value>
        /// An <see cref="IndexInfo"/> instance, created with the assembly
        /// and file name of this class, along with a <see cref="SimpleIndexParameters"/>
        /// instance that holds the path to the file that the current instance is currently
        /// working against.
        /// </value>
        ///
        /// <exception cref="InvalidOperationException">
        /// <see cref="FilePath"/> is a null reference.
        /// </exception>
        public IndexInfo IndexInfo
        {
            get
            {
                if (FilePath == null)
                {
                    throw new InvalidOperationException("'FilePath' is a null reference");
                }

                // create a new IndexInfo with FilePath in its parameter
                IParameters idxParam = new SimpleIndexParameters(FilePath);
                return new IndexInfo(
                    this.GetType().Assembly.FullName,
                    this.GetType().FullName,
                    idxParam);
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="CultureInfo"/> which is used for string comparisons.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="CultureInfo"/> which is used for string comparisons.
        /// </value>
        ///
        /// <remarks>
        /// The default value is CultureInfo("en-US").
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// Setting to a null reference.
        /// </exception>
        /// <exception cref="InvalidOperationException">
        /// This document words are not empty.
        /// </exception>
        public CultureInfo Culture
        {
            get
            {
                return culture;
            }
            set
            {
                if (value == null)
                {
                    throw new ArgumentNullException("Culture", "'Culture' cannot be null");
                }
                if (words.Count != 0)
                {
                    throw new InvalidOperationException(
                        "Document words exist already, cannot change Culture");
                }
                culture = value;
            }
        }

        /// <summary>
        /// Gets or sets the <see cref="CompareOptions"/>, which is used in conjunction
        /// with the <see cref="Culture"/> property for string comparison options.
        /// </summary>
        ///
        /// <value>
        /// The <see cref="CompareOptions"/>, which is used in conjunction
        /// with the <see cref="Culture"/> property for string comparison options.
        /// </value>
        ///
        /// <remarks>
        /// The default value is CompareOptions.None.
        /// </remarks>
        ///
        /// <exception cref="InvalidOperationException">
        /// This document words are not empty.
        /// </exception>
        public CompareOptions CompareOptions
        {
            get
            {
                return compareOptions;
            }
            set
            {
                if (words.Count != 0)
                {
                    throw new InvalidOperationException(
                        "Document words exist already, cannot change CompareOptions");
                }
                compareOptions = value;
            }
        }

        /// <summary>
        /// Gets or sets the file path where XML is loaded from and saved to.
        /// </summary>
        ///
        /// <value>
        /// The file path where XML is loaded from and saved to.
        /// </value>
        ///
        /// <exception cref="ArgumentNullException">
        /// Setting to a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// Setting to an empty trimmed string or the file path is invalid
        /// (either it is malformed or it points to a directory).
        /// </exception>
        public string FilePath
        {
            get
            {
                return filePath;
            }
            set
            {
                Helper.ValidateFilePath(value, "FilePath");
                filePath = value;
            }
        }

        /// <summary>
        /// Gets or sets the number of sub files to create when saving the file.
        /// </summary>
        ///
        /// <value>
        /// An integer greater than or equal to zero (zero means no sub-files),
        /// but not greater than the number of words in the document.
        /// </value>
        ///
        /// <remarks>
        /// It is used to set how many sub files the save mechanism should break the words across.
        /// This value has to be at least zero, where zero indicates that no subfiles are used
        /// and all words are in the XML file created.
        /// This value also has to be less than or equal to the number of words in the document.
        /// </remarks>
        ///
        /// <exception cref="ArgumentOutOfRangeException">
        /// Setting to a value less than zero or greater than the number of words in the document.
        /// </exception>
        public int NumberOfSubFiles
        {
            get
            {
                return numberOfSubFiles;
            }
            set
            {
                if ((value < 0) || (value > words.Count))
                {
                    throw new ArgumentOutOfRangeException("NumberOfSubFiles", value, string.Format(
                        "'NumberOfSubFiles' must be between 0 and {0}", words.Count));
                }
                numberOfSubFiles = value;
            }
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlDocumentIndexSource"/>.
        /// </summary>
        ///
        /// <remarks>
        /// The instance will initially be blank.
        /// The <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        public XmlDocumentIndexSource()
        {
            words = SortedList.Synchronized(new SortedList(this));
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="XmlDocumentIndexSource"/> class.
        /// </summary>
        ///
        /// <param name="filePath">
        /// The file path to save to.
        /// </param>
        ///
        /// <remarks>
        /// Note that <see cref="Load"/> method should be used to load the actual data in
        /// for this class.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="filePath"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="filePath"/> is an empty trimmed string or is invalid
        /// (either it is malformed or it points to a directory).
        /// </exception>
        public XmlDocumentIndexSource(string filePath)
            : this()
        {
            // setter will validate the file path
            FilePath = filePath;
        }

        /// <summary>
        /// Get or sets a <see cref="IList"/> of positions (long) for a given word.
        /// </summary>
        ///
        /// <param name="word">
        /// The word to return or set the locations for.
        /// </param>
        ///
        /// <value>
        /// An <see cref="IList"/> of all locations (long) for the given word in this document
        /// or null if the word doesn't exist.
        /// </value>
        ///
        /// <remarks>
        /// Note that if a word already exists for the specified name, it will be replaced.
        /// If the given IList contains duplicate position, it'll be merged when saved.
        /// The word will be trimmed before stored.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="word"/> is a null reference or setting to a null value.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="word"/> is an empty trimmed string or
        /// the IList contains non-long/integer type.
        /// </exception>
        public IList this[string word]
        {
            get
            {
                if (word == null)
                {
                    throw new ArgumentNullException("word", "'word' cannot be null");
                }
                if (word.Trim().Length == 0)
                {
                    throw new ArgumentException("'word' cannot be an empty trimmed string", "word");
                }

                return (IList) words[word];
            }
            set
            {
                if (word == null)
                {
                    throw new ArgumentNullException("word", "'word' cannot be null");
                }
                string key = word.Trim();
                if (key.Length == 0)
                {
                    throw new ArgumentException("'word' cannot be an empty trimmed string", "word");
                }
                if (value == null)
                {
                    throw new ArgumentNullException("Position list cannot be null");
                }
                foreach (object o in value)
                {
                    // only long/integer is accepted
                    if (!Helper.IsLong(o))
                    {
                        throw new ArgumentException(
                            "Position list contains non-long/integer element");
                    }
                }

                words[key] = new ArrayList(value);
            }
        }

        /// <summary>
        /// Adds the given <see cref="WordInfo"/> to this document.
        /// </summary>
        ///
        /// <param name="wordInfo">
        /// The information about the word to be added to the persistent store.
        /// </param>
        ///
        /// <remarks>
        /// If the specified word doesn't exist, a new IList representing the position is created.
        /// Otherwise the position will be added to the list.
        /// Duplicate position will be merged when the file is saved.
        /// The word will be trimmed before added.
        /// </remarks>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="wordInfo"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// The given word is an empty trimmed string.
        /// </exception>
        public void Add(WordInfo wordInfo)
        {
            if (wordInfo == null)
            {
                throw new ArgumentNullException("wordInfo", "'wordInfo' cannot be null");
            }

            // get the position list (AE is checked by get-indexer)
            string key = wordInfo.Word.Trim();
            IList posList = this[key];
            if (posList == null)
            {
                // create a new list and store it
                posList = new ArrayList();
                words[key] = posList;
            }

            // add the position to the list
            posList.Add(wordInfo.Position);
        }

        /// <summary>
        /// Removes the given word and its locations from the words collection.
        /// </summary>
        ///
        /// <remarks>
        /// Nothing will occur if the specified word doesn't exist.
        /// </remarks>
        ///
        /// <param name="word">
        /// The word to remove from the persistent store.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="word"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="word"/> is an empty trimmed string.
        /// </exception>
        public void Remove(string word)
        {
            if (word == null)
            {
                throw new ArgumentNullException("word", "'word' cannot be null");
            }
            if (word.Trim().Length == 0)
            {
                throw new ArgumentException("'word' cannot be an empty trimmed string", "indexName");
            }

            words.Remove(word);
        }

        /// <summary>
        /// Determines whether a word exists in the document.
        /// </summary>
        ///
        /// <param name="word">
        /// The word to look for.
        /// </param>
        ///
        /// <returns>
        /// true if the document contains the given word; otherwise, false.
        /// There has to be at least 1 position defined for the given word, otherwise
        /// false is always returned.
        /// </returns>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="word"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="word"/> is an empty trimmed string.
        /// </exception>
        public bool Contains(string word)
        {
            // getter check ANE/AE
            IList pos = this[word];

            return ((pos == null) ? false : (pos.Count > 0));
        }

        /// <summary>
        /// Clears all words from the document.
        /// </summary>
        public void Clear()
        {
            words.Clear();
        }

        /// <summary>
        /// Returns the enumerator for the words collection, which can be used by the user
        /// to enumerate through all words in this document.
        /// </summary>
        ///
        /// <returns>
        /// The enumerator for all words in the index.
        /// </returns>
        public IDictionaryEnumerator GetEnumerator()
        {
            return words.GetEnumerator();
        }

        /// <summary>
        /// Loads the document index from an Xml file.
        /// </summary>
        ///
        /// <remarks>
        /// The parameter given should be a <see cref="SimpleIndexParameters"/> instance
        /// that contains a file path that is used to initialize the filePath property.
        /// This method will take care to follow external links in the XML file, if the document persisted
        /// was large and has spanned multiple files.
        /// </remarks>
        ///
        /// <param name="parameters">
        /// The <see cref="SimpleIndexParameters"/> containing the xml file path to load.
        /// </param>
        ///
        /// <exception cref="ArgumentNullException">
        /// <paramref name="parameters"/> is a null reference.
        /// </exception>
        /// <exception cref="ArgumentException">
        /// <paramref name="parameters"/> contains an empty trimmed path or
        /// is not a <see cref="SimpleIndexParameters"/> instance or
        /// contains invalid FileName parameters
        /// (malformed, points to a directory, doesn't exists).
        /// </exception>
        /// <exception cref="LoadIndexDataException">
        /// Any errors occur while loading the Xml.
        /// </exception>
        public void Load(IParameters parameters)
        {
            if (parameters == null)
            {
                throw new ArgumentNullException("parameters", "'parameters' cannot be null");
            }
            if (!(parameters is SimpleIndexParameters))
            {
                throw new ArgumentException("'parameters' must be of SimpleIndexParameters type", "parameters");
            }

            // filepath must be valid (and save it to member var)
            FilePath = ((SimpleIndexParameters) parameters).FileName;
            // file not exists, throw AE
            if (!(File.Exists(FilePath)))
            {
                throw new ArgumentException(string.Format(
                    "FilePath {0} doesn't exist", FilePath));
            }

            // loads the xml file, and parse it (build words IDictionary)
            XmlDocument doc = LoadAndValidateXml(parameters);
            ParseXmlDoc(parameters, doc);
        }

        /// <summary>
        /// Saves the words indices to an Xml file.
        /// </summary>
        ///
        /// <remarks>
        /// The xml will conform to the Document schema as specified in the class documentation.
        /// If <see cref="NumberOfSubFiles"/> is greater than 0, the words are split into that many groups,
        /// with each group going to a different xml file (see Xml schema for Words).
        /// The filePath for each sub file is the name of the file specified in the <see cref="FilePath"/>,
        /// appended with a GUID, with an extension of "xml".
        /// The group size is calculated by taking the number of words and dividing that number
        /// by the number of subfiles to create. Any remaining entries will be stored in the last file.
        /// </remarks>
        ///
        /// <exception cref="InvalidOperationException">
        /// <see cref="FilePath"/> is a null reference.
        /// </exception>
        /// <exception cref="SaveIndexDataException">
        /// Any errors occur while saving the data.
        /// </exception>
        public void Save()
        {
            if (FilePath == null)
            {
                throw new InvalidOperationException("'FilePath' is a null reference");
            }

            try
            {
                XmlTextWriter w = null;
                try
                {
                    // before overwriting the file, retrieve all old-subfiles first (if exists)
                    IList subfiles = GetSubFiles(FilePath);

                    // instead of XmlDocument, XmlTextWriter provides faster, forward-only xml writing
                    w = new XmlTextWriter(FilePath, new UTF8Encoding());

                    // use the indented formatter, adhere to our coding standard of 4 spaces
                    w.Formatting = Formatting.Indented;
                    w.Indentation = Helper.XmlIndentation;

                    // write the XML declaration with version 1.0
                    w.WriteStartDocument();

                    // lv 1: <Document>
                    w.WriteStartElement(DocumentElement);

                    // lv 2: <CompareOptions>...</CompareOptions>
                    //       <Culture>...</Culture>
                    w.WriteElementString(CompareOptionsElement, CompareOptions.ToString());
                    w.WriteElementString(CultureElement, Culture.Name);

                    // lv 2: <Words>
                    w.WriteStartElement(WordsElement);

                    if (NumberOfSubFiles == 0)
                    {
                        // single file: create xml for the whole words dictionary
                        XmlNode wordsList = WordsListToXml(words);
                        w.WriteNode(new XmlNodeReader(wordsList), false);
                    }
                    else
                    {
                        // split words into chunks
                        IList chunks = SplitWords();

                        if (chunks != null)
                        {
                            // lv 3: <Files>
                            w.WriteStartElement(FilesElement);

                            // multiple file; break words into manageable chunks
                            foreach (IDictionary wordsChunk in chunks)
                            {
                                // generate a unique file name and save chunk as xml
                                string sub = GetUniqueSubFileName();
                                SaveWordsToFile(wordsChunk, sub);

                                // lv 4: <FilePath>...</FilePath>
                                w.WriteElementString(FilePathElement, sub);
                            }

                            // lv 3: </Files>
                            w.WriteEndElement();
                        }
                    }

                    // lv 2: </Words>
                    w.WriteEndElement();

                    // lv 1: </Document>
                    w.WriteEndElement();

                    // delete all old-subfiles
                    DeleteSubFiles(subfiles);
                }
                finally
                {
                    if (w != null)
                    {
                        // this may throw Exception, which will be reencapsulated into SaveIndexDataException
                        w.Close();
                    }
                }
            }
            catch (SaveIndexDataException)
            {
                // just re-throw
                throw;
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error saving to '{0}'", FilePath));
            }
        }

        /// <summary>
        /// Compare two words using the culture info contained in this class.
        /// </summary>
        ///
        /// <remarks>
        /// This method is used by the sort the word indices in this document.
        /// The <see cref="CompareOptions"/> is also taken into account.
        /// </remarks>
        /// <param name="x">
        /// The first object to compare.
        /// </param>
        /// <param name="y">
        /// The second object to compare.
        /// </param>
        ///
        /// <returns>
        /// Result of the comparison.
        /// </returns>
        ///
        /// <exception cref="ArgumentException">
        /// Either parameter is not a string.
        /// </exception>
        public int Compare(object x, object y)
        {
            if (!(x is string))
            {
                throw new ArgumentException("'x' must be of string type", "x");
            }
            if (!(y is string))
            {
                throw new ArgumentException("'y' must be of string type", "y");
            }
            // compare using the Culture and CompareOptions
            return Culture.CompareInfo.Compare((string) x, (string) y, CompareOptions);
        }

        /// <summary>
        /// Get words from a specific file path.
        /// </summary>
        ///
        /// <remarks>
        /// This method is used if we are loading a document index that is split over multiple files.
        /// This method can be used to take one of the split files and parse its words out into a return value.
        /// The IDictionary result has the word as the key, and the value is an IList of locations,
        /// which matches the format of the words IDictionary.
        /// </remarks>
        ///
        /// <param name="filePath">
        /// Xml File to load words from.
        /// </param>
        ///
        /// <returns>
        /// Words loaded from the given file path.
        /// </returns>
        ///
        /// <exception cref="LoadIndexDataException">
        /// The file can't be parsed correctly.
        /// </exception>
        private IDictionary GetWordsFromFile(string filePath)
        {
            try
            {
                // load the xml into DOM
                XmlDocument doc = Helper.LoadAndValidateXml(filePath, WordsXmlSchema);
                return XmlToWordsList(doc.DocumentElement);
            }
            catch (Exception e)
            {
                throw new LoadIndexDataException(new SimpleIndexParameters(filePath), e, string.Format(
                    "Error loading from subfile '{0}'", filePath));
            }
        }

        /// <summary>
        /// Writes the given words to an XML reference by the given file path.
        /// </summary>
        ///
        /// <remarks>
        /// This document will follow the Words.xsd schema.
        /// Each key in the dictionary is a word string, and
        /// the value is an IList of locations, the same format as the words member variable.
        /// </remarks>
        ///
        /// <param name="words">
        /// The words to save.
        /// </param>
        /// <param name="filePath">
        /// Where to save the words to.
        /// </param>
        ///
        /// <exception cref="SaveIndexDataException">
        /// An error occurs while saving the data.
        /// </exception>
        private void SaveWordsToFile(IDictionary words, string filePath)
        {
            try
            {
                XmlTextWriter w = null;
                try
                {
                    // instead of XmlDocument, XmlTextWriter provides faster, forward-only xml writing
                    w = new XmlTextWriter(filePath, new UTF8Encoding());

                    // use the indented formatter, adhere to our coding standard of 4 spaces
                    w.Formatting = Formatting.Indented;
                    w.Indentation = Helper.XmlIndentation;

                    // write the XML declaration with version 1.0
                    w.WriteStartDocument();

                    // like a single file: create xml for the whole words dictionary
                    XmlNode wordsList = WordsListToXml(words);
                    w.WriteNode(new XmlNodeReader(wordsList), false);
                }
                finally
                {
                    if (w != null)
                    {
                        // this may throw Exception, which will be reencapsulated into SaveIndexDataException
                        w.Close();
                    }
                }
            }
            catch (Exception e)
            {
                throw new SaveIndexDataException(e, string.Format(
                    "Error saving to subfile '{0}'", filePath));
            }
        }

        /// <summary>
        /// Loads and validates FilePath into an XmlDocument.
        /// </summary>
        ///
        /// <remarks>
        /// The file is loaded using <see cref="XmlValidatingReader"/> using the Document.xsd schema.
        /// The schema will provide xml structure validation.
        /// </remarks>
        ///
        /// <param name="param">
        /// The original <see cref="IParameters"/> used only for exception parameter.
        /// </param>
        ///
        /// <returns>
        /// A validated <see cref="XmlDocument"/>.
        /// </returns>
        ///
        /// <exception cref="LoadIndexDataException">
        /// Error loading and validating the xml file.
        /// </exception>
        private XmlDocument LoadAndValidateXml(IParameters param)
        {
            try
            {
                return Helper.LoadAndValidateXml(FilePath, DocumentXmlSchema);
            }
            catch (Exception e)
            {
                throw new LoadIndexDataException(param, e, string.Format(
                    "Error loading and validating xml file '{0}': {1}", FilePath, e.Message));
            }
        }

        /// <summary>
        /// Parses the specified XmlDocument (representing a Document.xsd)
        /// and build the indices IDictionary.
        /// </summary>
        ///
        /// <param name="param">
        /// The original <see cref="IParameters"/> used only for exception parameter.
        /// </param>
        /// <param name="xmlDoc">
        /// The <see cref="XmlDocument"/> to parse.
        /// </param>
        ///
        /// <exception cref="LoadIndexDataException">
        /// Error parsing the xml file.
        /// </exception>
        private void ParseXmlDoc(IParameters param, XmlDocument xmlDoc)
        {
            // save old words first (rollback to this when any error occurs)
            IDictionary oldWords = words;
            try
            {
                // clear words by creating a new one
                words = SortedList.Synchronized(new SortedList(this));

                // CompareOptions
                string str = xmlDoc.SelectSingleNode(XPathSelectCompareOptions).InnerText.Trim();
                CompareOptions = (CompareOptions) CompareOptions.Parse(typeof(CompareOptions), str);

                // Culture
                str = xmlDoc.SelectSingleNode(XPathSelectCulture).InnerText.Trim();
                Culture = new CultureInfo(str);

                // select WordsList node and process it
                XmlNode wordsListNode = xmlDoc.SelectSingleNode(XPathSelectWordsList);
                if (wordsListNode != null)
                {
                    IDictionary loadedWords = XmlToWordsList(wordsListNode);
                    AddToWordsIndices(loadedWords);
                }

                // select SubFiles and process each
                XmlNodeList subFilesNodes = xmlDoc.SelectNodes(XPathSelectSubFiles);
                foreach (XmlNode subFileNode in subFilesNodes)
                {
                    IDictionary loadedSubFiles = GetWordsFromFile(subFileNode.InnerText.Trim());
                    AddToWordsIndices(loadedSubFiles);
                }
            }
            catch (Exception e)
            {
                // roll-back (so we're not left with half-loaded data)
                words = oldWords;
                throw new LoadIndexDataException(param, e, string.Format(
                    "Error parsing xml file '{0}'", FilePath));
            }
        }

        /// <summary>
        /// Writes the given words IDictionary into xml nodes as specified by Words.xsd.
        /// If the word has no position defined, it won't be written to the xml file.
        /// If there's no word written, the returned XmlNode will be empty tag
        /// of <![CDATA[<WordsList><Word/><Locations><Location/></Locations></WordsList>]]>
        /// in order to conform to the Words schema.
        /// </summary>
        ///
        /// <param name="words">
        /// The words whose xml to write.
        /// </param>
        ///
        /// <returns>
        /// The root node of the resulting xml.
        /// </returns>
        private XmlNode WordsListToXml(IDictionary words)
        {
            // write xml to an XmlDocument
            XmlDocument doc = new XmlDocument();

            // lv 1: <WordsList>
            XmlElement wordsListElm = doc.CreateElement(WordsListElement);
            doc.AppendChild(wordsListElm);

            foreach (DictionaryEntry word in words)
            {
                // skip it if no position stored
                IList positions = (IList) word.Value;
                if (positions.Count == 0)
                {
                    continue;
                }

                // lv 2: <Word> and attach to <WordsList>
                XmlElement wordElm  = doc.CreateElement(WordElement);
                wordElm.InnerText = (string) word.Key;
                wordsListElm.AppendChild(wordElm);

                // lv 2: <Locations> and attach to <WordsList>
                XmlElement locationsElm = doc.CreateElement(LocationsElement);
                wordsListElm.AppendChild(locationsElm);

                // iterate positions, create <Location> and attach it (removing duplicates)
                IDictionary seen = new Hashtable();
                foreach (object posObject in positions)
                {
                    // positions may contain int/long
                    long pos = Convert.ToInt64(posObject);

                    // don't store duplicate (seen) values
                    if (!(seen.Contains(pos)))
                    {
                        // lv 3: <Location> and attach to <Locations>
                        XmlElement locationElm = doc.CreateElement(LocationElement);
                        locationElm.InnerText = pos.ToString();
                        locationsElm.AppendChild(locationElm);

                        // mark as seen
                        seen.Add(pos, null);
                    }
                }
            }

            // return an XmlNode to the root element
            return doc.DocumentElement;
        }

        /// <summary>
        /// Parses the given XmlNode which should be a WordsList node
        /// into an IDictionary of word string + IList of location.
        /// </summary>
        ///
        /// <param name="node">
        /// The xml node to parse.
        /// </param>
        ///
        /// <returns>
        /// The parsed word indices as an IDictionary.
        /// </returns>
        private IDictionary XmlToWordsList(XmlNode node)
        {
            IDictionary result = new SortedList(this);

            // iterate Word and Locations nodes
            XmlNodeList wordLocations = node.ChildNodes;
            IEnumerator it = wordLocations.GetEnumerator();
            while (it.MoveNext())
            {
                // get <Word> & <Locations>
                XmlNode wordElement = (XmlNode) it.Current;
                it.MoveNext();
                XmlNode locationsElement = (XmlNode) it.Current;

                // build IList from <Location>
                IList locs = new ArrayList();
                XmlNodeList locationsNode = locationsElement.ChildNodes;
                foreach (XmlNode locationNode in locationsNode)
                {
                    locs.Add(long.Parse(locationNode.InnerText.Trim()));
                }

                // add/merge to result IDictionary
                MergeValues(result, wordElement.InnerText.Trim(), locs);
            }
            return result;
        }

        /// <summary>
        /// Retrieves all registered sub-files from the current file.
        /// </summary>
        ///
        /// <param name="file">
        /// The xml file whose sub-files name to retrieve.
        /// </param>
        ///
        /// <returns>
        /// An IList containing sub-file names (could be empty).
        /// </returns>
        private IList GetSubFiles(string file)
        {
            IList subfiles = new ArrayList();

            try
            {
                // only if the file exists
                if (File.Exists(file))
                {
                    // select all FilePath and add to List
                    XmlDocument doc = LoadAndValidateXml(new SimpleIndexParameters(file));
                    XmlNodeList subFilesElement = doc.SelectNodes(XPathSelectSubFiles);
                    foreach (XmlNode subFileElement in subFilesElement)
                    {
                        subfiles.Add(subFileElement.InnerText.Trim());
                    }
                }
            }
            catch (Exception)
            {
                // any error means no sub-files
            }
            return subfiles;
        }

        /// <summary>
        /// Deletes all sub files as specified in the given list.
        /// </summary>
        ///
        /// <param name="subfiles">
        /// The sub files to delete.
        /// </param>
        private void DeleteSubFiles(IList subfiles)
        {
            foreach (string subfile in subfiles)
            {
                File.Delete(subfile);
            }
        }

        /// <summary>
        /// Splits words IDictionary into chunks as specified by NumberOfFiles.
        /// </summary>
        ///
        /// <remarks>
        /// If NumberOfSubFiles is greater than the number of words,
        /// the number of words is used instead.
        /// This method assume that NumberOfSubFiles always greater than 0.
        /// </remarks>
        ///
        /// <returns>
        /// An array of IDictionary of words chunks or null if no sub file is needed (no word to write).
        /// </returns>
        private IDictionary[] SplitWords()
        {
            // prepare array of chunks
            int noOfFiles = Math.Min(words.Count, NumberOfSubFiles);

            // no sub files needed (because words.Count = 0)
            if (noOfFiles == 0)
            {
                return null;
            }

            IDictionary[] result = new IDictionary[noOfFiles];
            // decide how many entries in each chunk
            // distribute the remaining in the first few files
            int entryEachChunk = words.Count / noOfFiles;
            int remaining = words.Count % noOfFiles;

            IDictionaryEnumerator word = GetEnumerator();
            for (int i = 0; i < noOfFiles; i++, remaining--)
            {
                result[i] = new SortedList(this);
                // if we have a remainder, add 1 more entry to this file
                for (int j = 0; j < entryEachChunk + ((remaining > 0) ? 1:0); j++)
                {
                    if (word.MoveNext())
                    {
                        result[i].Add(word.Key, word.Value);
                    }
                }
            }
            return result;
        }

        /// <summary>
        /// Generates a unique sub file name from the FilePath.
        /// </summary>
        ///
        /// <returns>
        /// A unique sub file name.
        /// </returns>
        private string GetUniqueSubFileName()
        {
            // get the dir and filename (without extension)
            string dirName = Path.GetDirectoryName(FilePath);
            if (dirName == null)
            {
                dirName = string.Empty;
            }
            string fileName = Path.GetFileNameWithoutExtension(FilePath);

            // build a new file name with GUID
            return string.Format(UniqueSubFileName, Path.Combine(dirName, fileName),
                Guid.NewGuid().ToString());
        }

        /// <summary>
        /// Adds/merges the given words IDictionary to the member variable words.
        /// </summary>
        ///
        /// <param name="wordsToMerge">
        /// The words IDictionary to add.
        /// </param>
        ///
        /// <exception cref="ArgumentException">
        /// If the given dictionary contains any empty trimmed word.
        /// </exception>
        private void AddToWordsIndices(IDictionary wordsToMerge)
        {
            foreach (DictionaryEntry word in wordsToMerge)
            {
                string key = (string) word.Key;
                if (key.Trim().Length == 0)
                {
                    throw new ArgumentException("Word cannot be an empty trimmed string");
                }
                IList positions = (IList) word.Value;
                MergeValues(this.words, key, positions);
            }
        }

        /// <summary>
        /// Merges the given key and position into words IDictionary.
        /// If the given key doesn't exists, a new List is created;
        /// otherwise the given list will be appended to existing one.
        /// </summary>
        ///
        /// <param name="wordsDict">
        /// The IDictionary to add/merge to.
        /// </param>
        /// <param name="key">
        /// The new word to add/merge.
        /// </param>
        /// <param name="positions">
        /// The IList of positions to add/merge.
        /// </param>
        private static void MergeValues(IDictionary wordsDict, string key, IList positions)
        {
            // get the position list
            IList posList = (IList) wordsDict[key];
            if (posList == null)
            {
                // create a new list and store it
                posList = new ArrayList();
                wordsDict[key] = posList;
            }

            // add the positions IList to the list
            foreach (object o in positions)
            {
                posList.Add(o);
            }
        }
    }
}
