/*
 * TestDBCollectionIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Data;
using NUnit.Framework;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Document.Index.Persistence.Tests;

namespace TopCoder.Document.Index.Persistence.DB.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DBCollectionIndexSource"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Save() method is used throughout the unit test cases to commit
    /// the data so that an assertion against the database can be performed.
    /// </remarks>
    [TestFixture]
    public class TestDBCollectionIndexSource : TestCommon
    {
        /// <summary>
        /// The <see cref="DBCollectionIndexSource"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private DBCollectionIndexSource source;

        /// <summary>
        /// The assembly name for DBDocumentIndexSource
        /// </summary>
        private static string assemblyName = typeof(DBDocumentIndexSource).Assembly.GetName().Name;

        /// <summary>
        /// The class name for DBDocumentIndexSource.
        /// </summary>
        private const string className = "TopCoder.Document.Index.Persistence.DB.DBDocumentIndexSource";

        /// <summary>
        /// Sample IndexInfo.
        /// Initialized in SetUp().
        /// </summary>
        private IndexInfo indexInfo;

        /// <summary>
        /// A database connection used by test cases to assert and prepares database.
        /// Initialized in SetUp() and closed in TearDown().
        /// </summary>
        private IDbConnection conn;

        /// <summary>
        /// Sql to deletes all rows from the document table.
        /// </summary>
        private SqlStatement clearsData = new SqlStatement("DELETE FROM Document");

        /// <summary>
        /// Initializes the <see cref="DBCollectionIndexSource"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // Connection Factory and DBCollectionIndexSource configs
            base.LoadConfigFile(ConnectionFactoryConfig,
                @"..\..\test_files\configs\DBCollectionIndexSource.xml");

            // init connection and cleans-up database
            conn = ConnectionManager.Instance.CreateDbConnection(DbConnectionName, new Hashtable());
            conn.Open();
            DeletesAllData();

            // prepare a sample IndexInfo (id = 123)
            indexInfo = new IndexInfo(assemblyName, className,
                new DBDocumentIndexParameters(DbConnectionName, 123));

            // prepare an empty DBCollectionIndexSource
            source = new DBCollectionIndexSource();
        }

        /// <summary>
        /// Cleans up configuration namespaces and deletes all rows from the database.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            // dispose instance (this will roll back and close the db connection)
            source.Dispose();

            // deletes all records and close our own db connection
            DeletesAllData();
            conn.Close();

            // clears up config manager namespace
            base.LoadConfigFile();
        }

        /// <summary>
        /// Deletes all rows from the database.
        /// </summary>
        private void DeletesAllData()
        {
            clearsData.ExecuteNonQuery(conn, null);
        }

        /// <summary>
        /// Tests constructor() accuracy and that
        /// no index is stored (which is tested via GetEnumerator and indexer).
        /// </summary>
        [Test]
        public void TestCtor()
        {
            // test via GetEnumerator
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

            // test via this indexer
            Assert.IsNull(source["Doc0"], "Doc not exists");
            Assert.IsNull(source["Doc1"], "Doc1 not exists");
        }

        /// <summary>
        /// Tests constructor(string) accuracy given valid namespace
        /// assert no index is stored (which is tested via GetEnumerator and indexer) initially.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // prepare a DBCollectionIndexSource given valid namespace
            using (DBCollectionIndexSource source =
                new DBCollectionIndexSource("TopCoder.Document.Index.Persistence.DB"))
            {
                // test via GetEnumerator
                IDictionaryEnumerator it = source.GetEnumerator();
                Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

                // test via this indexer
                Assert.IsNull(source["Doc0"], "Doc not exists");
                Assert.IsNull(source["Doc1"], "Doc1 not exists");
            }
        }

        /// <summary>
        /// Tests constructor(string) given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorStringNull()
        {
            new DBCollectionIndexSource(null);
        }

        /// <summary>
        /// Tests constructor(string) given empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new DBCollectionIndexSource(" ");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with missing DBConnectionName.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringMissing()
        {
            new DBCollectionIndexSource("TopCoder.Document.Index.Persistence.DB.Missing");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with empty DBConnectionName.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringEmptyDBConnectionName()
        {
            new DBCollectionIndexSource("TopCoder.Document.Index.Persistence.DB.Empty");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with invalid DBConnectionName.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringInvalid()
        {
            new DBCollectionIndexSource("TopCoder.Document.Index.Persistence.DB.Invalid");
        }

        /// <summary>
        /// Tests Save() and indexer setter and getter accuracy.
        /// </summary>
        [Test]
        public void TestIndexer()
        {
            // insert into the database and commits
            source["Doc0"] = indexInfo;
            source.Save();
            Assert.AreSame(indexInfo, source["Doc0"]);

            using (IDataReader reader = new SqlStatement(
                       "SELECT Document_Id, Document_Name, Compare_Options, Culture FROM Document")
                       .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "1 row is inserted");
                Assert.IsTrue(reader.GetInt64(0) > 0, "A new ID is assigned");
                Assert.AreEqual("Doc0", reader.GetString(1), "The document name");
                Assert.AreEqual("None", reader.GetString(2), "Default CompareOptions");
                Assert.AreEqual("en-US", reader.GetString(3), "Default Culture");
                Assert.IsFalse(reader.Read(), "No more row");
            }

            Assert.IsNull(source["NotExists"], "Not exists");
        }

        /// <summary>
        /// Tests indexer trims the index name.
        /// </summary>
        [Test]
        public void TestIndexerTrim()
        {
            // add index info with spaces that should be trimmed
            source[" Trim Test  "] = indexInfo;
            source.Save();

            IndexInfo assert = source["Trim Test"];
            Assert.AreSame(indexInfo, assert, "Same object");

            // reload and assert again
            source.Load(new DBPersistenceParameters(DbConnectionName));
            assert = source["Trim Test"];
            Assert.IsTrue(assert.AssemblyName.StartsWith(assemblyName), "AssemblyName is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName is incorrect");
        }

        /// <summary>
        /// Tests get-indexer given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerGetNull()
        {
            IndexInfo idx = source[null];
        }

        /// <summary>
        /// Tests get-indexer given empty index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerGetEmpty()
        {
            IndexInfo idx = source[" "];
        }

        /// <summary>
        /// Tests set-indexer replaces existing IndexInfo given the same name.
        /// </summary>
        [Test]
        public void TestIndexerSet()
        {
            // create a new index info
            IndexInfo idx = new IndexInfo(assemblyName, className,
                new DBDocumentIndexParameters(DbConnectionName, 12345));

            // insert first, replace, and commits
            source["Doc1"] = indexInfo;
            source["Doc1"] = idx;
            source.Save();
            Assert.AreSame(indexInfo, source["Doc1"]);

            // assert that only 1 document is inserted
            using (IDataReader reader = new SqlStatement(
                       "SELECT Document_Id, Document_Name, Compare_Options, Culture FROM Document")
                       .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "1 row is inserted");
                Assert.IsTrue(reader.GetInt64(0) > 0, "A new ID is assigned");
                Assert.AreEqual("Doc1", reader.GetString(1), "The document name");
                Assert.AreEqual("None", reader.GetString(2), "Default CompareOptions");
                Assert.AreEqual("en-US", reader.GetString(3), "Default Culture");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests set-indexer given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullName()
        {
            source[null] = indexInfo;
        }

        /// <summary>
        /// Tests set-indexer given empty index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerSetEmpty()
        {
            source[" "] = indexInfo;
        }

        /// <summary>
        /// Tests set-indexer given null IndexInfo.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullIndexInfo()
        {
            source["testNull"] = null;
        }

        /// <summary>
        /// Tests Add accuracy by adding a new index info (name is case-sensitive).
        /// </summary>
        [Test]
        public void TestAdd()
        {
            source.Add("doc0", indexInfo);
            source.Save();
            Assert.IsNotNull(source["doc0"], "Indices updated");

            // assert that only 1 document is inserted
            using (IDataReader reader = new SqlStatement(
                       "SELECT Document_Id, Document_Name, Compare_Options, Culture FROM Document")
                       .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "1 row is inserted");
                Assert.IsTrue(reader.GetInt64(0) > 0, "A new ID is assigned");
                Assert.AreEqual("doc0", reader.GetString(1), "The document name");
                Assert.AreEqual("None", reader.GetString(2), "Default CompareOptions");
                Assert.AreEqual("en-US", reader.GetString(3), "Default Culture");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests Add trim the word.
        /// </summary>
        [Test]
        public void TestAddTrim()
        {
            // add index info with spaces that should be trimmed
            source.Add(" Trim Test  ", indexInfo);
            source.Save();

            IndexInfo assert = source["Trim Test"];
            Assert.AreSame(indexInfo, assert, "Same object");

            // reload and assert again
            source.Load(new DBPersistenceParameters(DbConnectionName));
            assert = source["Trim Test"];
            Assert.IsTrue(assert.AssemblyName.StartsWith(assemblyName), "AssemblyName is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName is incorrect");
        }

        /// <summary>
        /// Tests Add given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullName()
        {
            source.Add(null, indexInfo);
        }

        /// <summary>
        /// Tests Add given empty index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddEmpty()
        {
            source.Add(" ", indexInfo);
        }

        /// <summary>
        /// Tests Add given null IndexInfo.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullIndexInfo()
        {
            source.Add("testNull", null);
        }

        /// <summary>
        /// Tests Add given existing index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddExist()
        {
            // add first entry
            source.Add("Doc1", indexInfo);
            Assert.IsNotNull(source["Doc1"], "Indices updated");

            // adding existing entry throws exception
            source.Add("Doc1", new IndexInfo("a", "b",
                new DBPersistenceParameters(DbConnectionName)));
        }

        /// <summary>
        /// Tests Add given IndexInfo with some arbitrary AssemblyName.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddAssembly()
        {
            source.Add("Doc", new IndexInfo("aa", className,
                new DBDocumentIndexParameters(DbConnectionName, 1)));
        }

        /// <summary>
        /// Tests Add given IndexInfo with some arbitrary ClassName.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddClass()
        {
            source.Add("Doc", new IndexInfo(assemblyName, "cc",
                new DBDocumentIndexParameters(DbConnectionName, 1)));
        }

        /// <summary>
        /// Tests Add given IndexInfo with non SimpleIndexParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddNonSimpleIndexParameters()
        {
            source.Add("Doc", new IndexInfo(assemblyName, className,
                new DBPersistenceParameters(DbConnectionName)));
        }

        /// <summary>
        /// Tests Remove accuracy by removing existing name.
        /// </summary>
        [Test]
        public void TestRemove()
        {
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Document"))
            {
                // add new entry
                source.Add("ToRemove", indexInfo);
                source.Add("Don'tRemove", indexInfo);
                source.Save();
                Assert.AreEqual(2, countSql.ExecuteScalar(conn, null), "2 rows inserted");

                // remove it
                source.Remove("ToRemove");
                source.Save();
                Assert.IsNull(source["ToRemove"], "Indices updated");
                Assert.AreEqual(1, countSql.ExecuteScalar(conn, null), "1 row deleted");
            }
        }

        /// <summary>
        /// Tests Remove by removing non-existing name (name is case-sensitive).
        /// Nothing should be removed.
        /// </summary>
        [Test]
        public void TestRemoveNotExist()
        {
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Document"))
            {
                // add new entry
                source.Add("Don'tRemove", indexInfo);
                source.Save();
                Assert.AreEqual(1, countSql.ExecuteScalar(conn, null), "1 row inserted");

                // remove some non-existing entry
                source.Remove("NotExists");
                source.Save();
                Assert.IsNotNull(source["Don'tRemove"], "Indices still exists");
                Assert.AreEqual(1, countSql.ExecuteScalar(conn, null), "1 row still exists");
            }
        }

        /// <summary>
        /// Tests Remove given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveNull()
        {
            source.Remove(null);
        }

        /// <summary>
        /// Tests Remove given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRemoveEmpty()
        {
            source.Remove(" ");
        }

        /// <summary>
        /// Tests GetEnumerator accuracy by iterating existing collection.
        /// </summary>
        [Test]
        public void TestGetEnumerator()
        {
            source.Add("Doc0", indexInfo);
            source.Add("Doc1", indexInfo);
            source.Save();
            IDictionaryEnumerator it = source.GetEnumerator();

            // 1st entry
            Assert.IsTrue(it.MoveNext(), "1st entry");
            Assert.AreEqual("Doc0", it.Key, "1st key");
            Assert.AreSame(indexInfo, it.Value, "1st value");

            // 2nd entry
            Assert.IsTrue(it.MoveNext(), "2nd entry");
            Assert.AreEqual("Doc1", it.Key, "2nd key");
            Assert.AreSame(indexInfo, it.Value, "2nd value");

            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Remove and GetEnumerator accuracy that an element is removed.
        /// </summary>
        [Test]
        public void TestGetEnumeratorRemove()
        {
            // add two elements then remove one
            source.Add("Doc0", indexInfo);
            source.Add("Doc1", indexInfo);
            source.Save();
            source.Remove("Doc0");
            source.Save();

            // reload data
            source.Load(new DBPersistenceParameters(DbConnectionName));
            IDictionaryEnumerator it = source.GetEnumerator();

            // we have 1 entry now
            Assert.IsTrue(it.MoveNext(), "1st entry");
            Assert.AreEqual("Doc1", it.Key, "1st key");
            Assert.IsFalse(indexInfo == it.Value, "Different instance");
            Assert.AreEqual(DbConnectionName,
                ((DBPersistenceParameters) ((IndexInfo) it.Value).Parameters).ConnectionName, "1st value");

            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Contains accuracy by checking existing and non-existing name.
        /// </summary>
        [Test]
        public void TestContains()
        {
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Document"))
            {
                // insert 2 entries
                source["Doc0"] = indexInfo;
                source["Doc1"] = indexInfo;
                source.Save();
                Assert.AreEqual(2, countSql.ExecuteScalar(conn, null), "2 records added");

                // assert with contains
                Assert.IsTrue(source.Contains("Doc0"), "1st entry");
                Assert.IsTrue(source.Contains("Doc1"), "2nd entry");
                Assert.IsFalse(source.Contains("doc1"), "Name is case-sensitive");
            }
        }

        /// <summary>
        /// Tests Contains given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestContainsNull()
        {
            source.Contains(null);
        }

        /// <summary>
        /// Tests Contains given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestContainsEmpty()
        {
            source.Contains(" ");
        }

        /// <summary>
        /// Tests Clear removes all collections.
        /// Assert that it is cleared via Contains, indexer, and GetEnumerator.
        /// </summary>
        [Test]
        public void TestClear()
        {
            // insert an entry into document
            source.Add("ToRemove", indexInfo);
            source.Save();

            // clears and asserts
            source.Clear();
            source.Save();
            Assert.IsFalse(source.Contains("ToRemove"), "No entry");
            Assert.IsNull(source["ToRemove"], "No index");
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No entry in enumerator");

            // assert the db
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Document"))
            {
                Assert.AreEqual(0, countSql.ExecuteScalar(conn, null));
            }
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Word"))
            {
                Assert.AreEqual(0, countSql.ExecuteScalar(conn, null));
            }
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Location"))
            {
                Assert.AreEqual(0, countSql.ExecuteScalar(conn, null));
            }
        }

        /// <summary>
        /// Tests Load accuracy by inserting some data and load it back.
        /// </summary>
        [Test]
        public void TestLoad()
        {
            // insert 2 records
            source.Add("doc0", indexInfo);
            source["New document"] = indexInfo;
            source.Save();

            // clear and reload
            source.Clear();
            Assert.IsNull(source["doc0"], "doc0 Cleared");
            Assert.IsNull(source["New document"], "New document Cleared");
            source.Load(new DBPersistenceParameters(DbConnectionName));

            // assert loaded data
            IndexInfo assert = source["doc0"];
            Assert.IsTrue(assert.AssemblyName.StartsWith(assemblyName), "AssemblyName is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName is incorrect");
            Assert.AreEqual(DbConnectionName, ((DBDocumentIndexParameters) assert.Parameters).ConnectionName,
                "Parameter.ConnectionName is incorrect");
            Assert.IsTrue(((DBDocumentIndexParameters) assert.Parameters).DocumentID > 0,
                "Parameter.DocumentId is incorrect");

            assert = source["New document"];
            Assert.IsTrue(assert.AssemblyName.StartsWith(assemblyName), "AssemblyName2 is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName2 is incorrect");
            Assert.AreEqual(DbConnectionName, ((DBDocumentIndexParameters) assert.Parameters).ConnectionName,
                "Parameter.ConnectionName2 is incorrect");
            Assert.IsTrue(((DBDocumentIndexParameters) assert.Parameters).DocumentID > 0,
                "Parameter.DocumentId2 is incorrect");

            // 2 records should be loaded
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsTrue(it.MoveNext(), "1st data is loaded");
            Assert.IsTrue(it.MoveNext(), "2nd data is loaded");
            Assert.IsFalse(it.MoveNext(), "No more data");
        }

        /// <summary>
        /// Tests Load roll back any pending uncommitted transaction.
        /// </summary>
        [Test]
        public void TestLoadRollback()
        {
            // insert 2 records (but don't save)
            source.Add("doc0", indexInfo);
            source["New document"] = indexInfo;

            // reload (this will clear everything)
            source.Load(new DBPersistenceParameters(DbConnectionName));
            Assert.IsNull(source["doc0"], "doc0 Cleared");
            Assert.IsNull(source["New document"], "New document Cleared");

            // no data because db is empty
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsFalse(it.MoveNext(), "No data is loaded");
        }

        /// <summary>
        /// Tests Load given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestLoadNull()
        {
            source.Load(null);
        }

        /// <summary>
        /// Tests Load given Parameter which is not DBPersistenceParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadNotDBPersistenceParameters()
        {
            source.Load(new MyParameters());
        }

        /// <summary>
        /// Tests Load given Parameter which is a DBDocumentIndexParameters.
        /// Expecting that the connection name is used.
        /// </summary>
        [Test]
        public void TestLoadDBDocumentIndexParameters()
        {
            // insert 1 document
            source.Add("Test<>", indexInfo);
            source.Save();

            // clear and reload with DBDocumentIndexParameters (docId is ignored)
            source.Clear();
            source.Load(new DBDocumentIndexParameters(DbConnectionName, 123));

            Assert.IsNotNull(source["Test<>"], "Document retrieved");
            Assert.IsFalse(source["Test<>"] == indexInfo, "Different instance");
        }

        /// <summary>
        /// Tests Load given Parameter that points to an invalid connection name.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestLoadInvalid()
        {
            source.Load(new DBPersistenceParameters("NotExists"));
        }

        /// <summary>
        /// Tests Save given a blank, no transaction, DBCollectionIndexSource.
        /// Nothing will happen.
        /// </summary>
        [Test]
        public void TestSaveNothing()
        {
            // save (this should do nothing)
            source.Save();

            // assert Document table is empty
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Document"))
            {
                Assert.AreEqual(0, countSql.ExecuteScalar(conn, null));
            }
        }

        /// <summary>
        /// Tests Save given collection with characters that must be escaped (in the xml),
        /// but not necessarily a problem for the database.
        /// </summary>
        [Test]
        public void TestSaveEscapedChars()
        {
            // string with quote, double quote, <, >
            source.Add("<Test'quote' \"Double quote\"/>", indexInfo);
            source.Save();

            // reloading the file should not raise exception
            source.Load(new DBPersistenceParameters(DbConnectionName));

            IndexInfo assert = source["<Test'quote' \"Double quote\"/>"];
            Assert.IsNotNull(assert, "Retrieved without any problem");
            Assert.IsFalse(assert == indexInfo, "Not the same instance");

            Assert.IsTrue(assert.AssemblyName.StartsWith(assemblyName), "AssemblyName is incorrect");
            Assert.AreEqual(className, assert.ClassName, "ClassName is incorrect");
            Assert.AreEqual(DbConnectionName, ((DBDocumentIndexParameters) assert.Parameters).ConnectionName,
                "Parameter.ConnectionName is incorrect");
            Assert.IsTrue(((DBDocumentIndexParameters) assert.Parameters).DocumentID > 0,
                "Parameter.DocumentId is incorrect");
        }
    }
}
