/*
 * TestDBDocumentIndexSource.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Data;
using System.Globalization;
using NUnit.Framework;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Document.Index.Persistence.Tests;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.DB.Tests
{
    /// <summary>
    /// Unit tests for the <see cref="DBDocumentIndexSource"/> class.
    /// </summary>
    ///
    /// <remarks>
    /// Save() method is used throughout the unit test cases to commit
    /// the data so that an assertion against the database can be performed.
    /// </remarks>
    [TestFixture]
    public class TestDBDocumentIndexSource : TestCommon
    {
        /// <summary>
        /// The <see cref="DBDocumentIndexSource"/> instance to test.
        /// Initialized in SetUp().
        /// </summary>
        private DBDocumentIndexSource source;

        /// <summary>
        /// The assembly name for DBDocumentIndexSource
        /// </summary>
        private const string assemblyName = "TopCoder.Document.Index.Persistence";

        /// <summary>
        /// The class name for DBDocumentIndexSource.
        /// </summary>
        private const string className = "TopCoder.Document.Index.Persistence.DB.DBDocumentIndexSource";

        /// <summary>
        /// A database connection used by test cases to assert and prepares database.
        /// Initialized in SetUp() and closed in TearDown().
        /// </summary>
        private IDbConnection conn;

        /// <summary>
        /// Sql to deletes all rows from the document/word/location table.
        /// </summary>
        private SqlStatement[] clearsData = new SqlStatement[]
            {
                new SqlStatement("DELETE FROM Location"),
                new SqlStatement("DELETE FROM Word"),
                new SqlStatement("DELETE FROM Document")
            };

        /// <summary>
        /// Sample document id. Initializes in PrepareSample().
        /// </summary>
        private int sampleDocumentId;

        /// <summary>
        /// Prepares parameter initialized with sample document id.
        /// Initialized in SetUp().
        /// </summary>
        private IParameters param;

        /// <summary>
        /// Initializes the <see cref="DBDocumentIndexSource"/> instance to test.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // Connection Factory and DBDocumentIndexSource configs
            base.LoadConfigFile(ConnectionFactoryConfig,
                @"..\..\test_files\configs\DBDocumentIndexSource.xml");

            // init connection and prepares sample data
            conn = ConnectionManager.Instance.CreateDbConnection(DbConnectionName, new Hashtable());
            conn.Open();
            DeletesAllData();
            PrepareSamples();

            // prepare an empty DBDocumentIndexSource and param
            source = new DBDocumentIndexSource();
            // load to force DocumentId to be loaded
            param = new DBDocumentIndexParameters(DbConnectionName, sampleDocumentId);
            source.Load(param);
        }

        /// <summary>
        /// Cleans up configuration namespaces and deletes all rows from the database.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            // dispose instance (this will roll back and close the db connection)
            source.Dispose();

            // deletes all records and close our own db connection
            DeletesAllData();
            conn.Close();

            // clears up config manager namespace
            base.LoadConfigFile();
        }

        /// <summary>
        /// Deletes all rows from the database.
        /// </summary>
        private void DeletesAllData()
        {
            foreach (SqlStatement stmt in clearsData)
            {
                stmt.ExecuteNonQuery(conn, null);
            }
        }

        /// <summary>
        /// Inserts some initial samples data into the database.
        /// </summary>
        private void PrepareSamples()
        {
            // insert Document
            new SqlStatement(
                "INSERT INTO Document (Document_Name, Compare_Options, Culture) "
                + "VALUES ('TCS Document', 'None', 'en-US')").ExecuteNonQuery(conn, null);
            sampleDocumentId = Convert.ToInt32(
                new SqlStatement("SELECT Document_Id FROM Document").ExecuteScalar(conn, null));

            // word is: TopCoder Open 2005 Document Index Persistence TopCoder TopCoder TopCoder Open".Split(' ');
            // 10 words: 6 unique
            string[] words = new string[]
                {"TopCoder", "Open", "2005", "Document", "Index", "Persistence"};
            int[][] pos = new int[][]
                {
                    new int[] {0, 6, 7, 8},
                    new int[] {1, 9},
                    new int[] {2},
                    new int[] {3},
                    new int[] {4},
                    new int[] {5}
                };

            // insert word and locations
            SqlStatement sqlInsertWord = new SqlStatement(
                "INSERT INTO Word (Word) VALUES (?)", "Word");
            SqlStatement sqlSelectWord = new SqlStatement(
                "SELECT Word_Id FROM Word WHERE Word = ?", "Word");
            SqlStatement sqlInsertLoc = new SqlStatement(
                "INSERT INTO Location (Word_Id, Document_Id, Location) VALUES (?, ?, ?)",
                "Word_Id", "Document_Id", "Location");

            for (int i = 0; i < words.Length; i++)
            {
                // insert word
                sqlInsertWord.ExecuteNonQuery(conn, null, words[i]);
                int wordId = Convert.ToInt32(
                    sqlSelectWord.ExecuteScalar(conn, null, words[i]));

                // insert locations
                foreach (int p in pos[i])
                {
                    sqlInsertLoc.ExecuteNonQuery(conn, null, wordId, sampleDocumentId, p);
                }
            }
        }

        /// <summary>
        /// Tests constructor() accuracy and that
        /// no word is stored (which is tested via GetEnumerator and indexer).
        /// </summary>
        [Test]
        public void TestCtor()
        {
            using (DBDocumentIndexSource source = new DBDocumentIndexSource())
            {
                // test via GetEnumerator
                IDictionaryEnumerator it = source.GetEnumerator();
                Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

                // test via this indexer
                Assert.IsNull(source["TopCoder"], "TopCoder not exists");
                Assert.IsNull(source["2005"], "2005 not exists");
            }
        }

        /// <summary>
        /// Tests constructor(string) accuracy given valid namespace
        /// assert no index is stored (which is tested via GetEnumerator and indexer) initially.
        /// </summary>
        [Test]
        public void TestCtorString()
        {
            // prepare a DBDocumentIndexSource given valid namespace (docid doesn't exist though)
            using (DBDocumentIndexSource source =
                new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB"))
            {
                // test via GetEnumerator
                IDictionaryEnumerator it = source.GetEnumerator();
                Assert.IsFalse(it.MoveNext(), "Nothing to enumerate");

                // test via this indexer
                Assert.IsNull(source["TopCoder"], "TopCoder not exists");
                Assert.IsNull(source["2005"], "2005 not exists");
            }
        }

        /// <summary>
        /// Tests constructor(String) given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCtorStringNull()
        {
            new DBDocumentIndexSource(null);
        }

        /// <summary>
        /// Tests constructor(String) given empty string argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCtorStringEmpty()
        {
            new DBDocumentIndexSource(" ");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with missing DBConnectionName.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringMissingDB()
        {
            new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB.MissingDB");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with missing DocumentID.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringMissingID()
        {
            new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB.MissingID");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with empty DBConnectionName.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringEmptyDBConnectionName()
        {
            new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB.EmptyDB");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with empty DocumentID.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringEmptyDocumentID()
        {
            new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB.EmptyID");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with invalid DBConnectionName.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringInvalidDB()
        {
            new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB.InvalidDB");
        }

        /// <summary>
        /// Tests constructor(string) given configuration with invalid DocumentID.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestCtorStringInvalidID()
        {
            new DBDocumentIndexSource("TopCoder.Document.Index.Persistence.DB.InvalidID");
        }

        /// <summary>
        /// Tests IndexInfo accuracy.
        /// </summary>
        [Test]
        public void TestIndexInfo()
        {
            IndexInfo idx = source.IndexInfo;
            Assert.IsTrue(idx.AssemblyName.StartsWith(assemblyName), "AssemblyName is incorrect");
            Assert.AreEqual(className, idx.ClassName, "ClassName is incorrect");
            DBDocumentIndexParameters param = (DBDocumentIndexParameters) idx.Parameters;
            Assert.AreEqual(DbConnectionName, param.ConnectionName, "Parameters.ConnectinoName is incorrect");
            Assert.AreEqual(sampleDocumentId, param.DocumentID, "Parameters.DocumentID is incorrect");
        }

        /// <summary>
        /// Tests Culture getter and setter accuracy given the document in SetUp().
        /// </summary>
        [Test]
        public void TestCulture()
        {
            Assert.AreEqual("en-US", source.Culture.Name, "Default Culture is en-us");

            // change culture
            source.Clear();
            source.Culture = new CultureInfo("pl-PL");
            source.Save();
            Assert.AreEqual("pl-PL", source.Culture.Name, "Culture changed");

            object culture = new SqlStatement(
                "SELECT Culture FROM Document WHERE Document_Id=" + sampleDocumentId)
                .ExecuteScalar(conn, null);
            Assert.AreEqual("pl-PL", culture, "Persisted");
        }

        /// <summary>
        /// Tests Culture getter and setter accuracy given a new blank document
        /// that has not been persisted.
        /// </summary>
        [Test]
        public void TestCultureNotExist()
        {
            // Culture on default empty instance
            using (DBDocumentIndexSource source = new DBDocumentIndexSource())
            {
                Assert.AreEqual("en-US", source.Culture.Name, "Default Culture is en-us");

                // the docid doesn't exists, so this should not be persisted
                source.Culture = new CultureInfo("pl-PL");
                source.Save();
                Assert.AreEqual("pl-PL", source.Culture.Name, "Culture changed");

                object culture = new SqlStatement(
                    "SELECT Culture FROM Document WHERE Document_Id="
                    + ((DBDocumentIndexParameters) source.IndexInfo.Parameters).DocumentID)
                    .ExecuteScalar(conn, null);
                Assert.IsNull(culture, "Not persisted");
            }
        }

        /// <summary>
        /// Tests Culture setter given null reference.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestCultureNull()
        {
            source.Culture = null;
        }

        /// <summary>
        /// Tests Culture setter when the words list is not empty.
        /// Expecting InvalidOperationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestCultureWordsNotEmpty()
        {
            source.Culture = new CultureInfo("ja-JP");
        }

        /// <summary>
        /// Tests CompareOptions getter and setter accuracy given the document in SetUp().
        /// </summary>
        [Test]
        public void TestCompareOptions()
        {
            Assert.AreEqual(CompareOptions.None, source.CompareOptions, "Default CompareOptions is None");

            // change CompareOptions
            source.Clear();
            source.CompareOptions = CompareOptions.StringSort;
            source.Save();
            Assert.AreEqual(CompareOptions.StringSort, source.CompareOptions, "CompareOptions changed");

            object compareOptions = new SqlStatement(
                "SELECT Compare_Options FROM Document WHERE Document_Id=" + sampleDocumentId)
                .ExecuteScalar(conn, null);
            Assert.AreEqual("StringSort", compareOptions, "Persisted");
        }

        /// <summary>
        /// Tests CompareOptions getter and setter accuracy given a new blank document
        /// that has not been persisted.
        /// </summary>
        [Test]
        public void TestCompareOptionsNotExist()
        {
            // CompareOptions on default empty instance
            using (DBDocumentIndexSource source = new DBDocumentIndexSource())
            {
                Assert.AreEqual(CompareOptions.None, source.CompareOptions, "Default CompareOptions is None");

                // the docid doesn't exists, so this should not be persisted
                source.CompareOptions = CompareOptions.IgnoreWidth;
                source.Save();
                Assert.AreEqual(CompareOptions.IgnoreWidth, source.CompareOptions, "CompareOptions changed");

                object compareOptions = new SqlStatement(
                    "SELECT Compare_Options FROM Document WHERE Document_Id="
                    + ((DBDocumentIndexParameters) source.IndexInfo.Parameters).DocumentID)
                    .ExecuteScalar(conn, null);
                Assert.IsNull(compareOptions, "Not persisted");
            }
        }

        /// <summary>
        /// Tests CompareOptions setter when the words list is not empty.
        /// Expecting InvalidOperationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(InvalidOperationException))]
        public void TestCompareOptionsWordsNotEmpty()
        {
            source.CompareOptions = CompareOptions.IgnoreWidth;
        }

        /// <summary>
        /// Tests get-indexer accuracy as initialized in SetUp().
        /// </summary>
        [Test]
        public void TestIndexerGet()
        {
            // 2005
            IList pos = source["2005"];
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            pos = source["Document"];
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            pos = source["Index"];
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            pos = source["Open"];
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            pos = source["Persistence"];
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            pos = source["TopCoder"];
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");

            Assert.IsNull(source["NotExists"], "Word not exists");
        }

        /// <summary>
        /// Tests get-indexer given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerGetNull()
        {
            IList pos = source[null];
        }

        /// <summary>
        /// Tests get-indexer given empty word.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerGetEmpty()
        {
            IList pos = source[" "];
        }

        /// <summary>
        /// Tests set-indexer accuracy by setting a new word.
        /// </summary>
        [Test]
        public void TestIndexerSet()
        {
            // create a new position list
            IList pos = new ArrayList();
            pos.Add(12345);
            pos.Add(11);
            pos.Add(0);
            source["new word !@<>"] = pos;
            source.Save();

            IList assert = source["new word !@<>"];
            Assert.AreEqual(3, assert.Count, "Assert count");
            Assert.AreEqual(12345, assert[0], "Assert 1st element");
            Assert.AreEqual(11, assert[1], "Assert 2nd element");
            Assert.AreEqual(0, assert[2], "Assert 3rd element");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='new word !@<>'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word is not persisted");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                 + " AND Word_Id = " + wordId
                 + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(0, reader.GetInt64(0), "First loc");
                Assert.IsTrue(reader.Read(), "Second row");
                Assert.AreEqual(11, reader.GetInt64(0), "Second loc");
                Assert.IsTrue(reader.Read(), "Third row");
                Assert.AreEqual(12345, reader.GetInt64(0), "Third loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests set-indexer given null word.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullWord()
        {
            IList pos = new ArrayList();
            pos.Add(10);
            source[null] = pos;
        }

        /// <summary>
        /// Tests set-indexer given empty word.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerSetEmpty()
        {
            IList pos = new ArrayList();
            pos.Add(10);
            source[" "] = pos;
        }

        /// <summary>
        /// Tests set-indexer given null IList.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestIndexerSetNullIList()
        {
            source["testNull"] = null;
        }

        /// <summary>
        /// Tests set-indexer given existing word.
        /// It should overwrite existing entry.
        /// </summary>
        [Test]
        public void TestIndexerSetExist()
        {
            IList newPos = new ArrayList();
            newPos.Add(1789);
            newPos.Add(1);
            source["2005"] = newPos;
            source.Save();

            IList assert = source["2005"];
            Assert.AreEqual(2, assert.Count, "Assert count");
            Assert.AreEqual(1789, assert[0], "Assert 1st element");
            Assert.AreEqual(1, assert[1], "Assert 2nd element");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='2005'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word still exists");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(1, reader.GetInt64(0), "First loc");
                Assert.IsTrue(reader.Read(), "Second row");
                Assert.AreEqual(1789, reader.GetInt64(0), "Second loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests set-indexer given empty List.
        /// It should overwrite existing entry.
        /// </summary>
        [Test]
        public void TestIndexerSetEmptyList()
        {
            source["Open"] = new ArrayList();
            source.Save();

            IList assert = source["Open"];
            Assert.AreEqual(0, assert.Count, "Assert count");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Open'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word still exists");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsFalse(reader.Read(), "No row");
            }
        }

        /// <summary>
        /// Tests set-indexer given non long/integer entry.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNonLong()
        {
            IList pos = new ArrayList();
            pos.Add('a');
            source["open"] = pos;
        }

        /// <summary>
        /// Tests set-indexer given null entry in the IList.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestIndexerNullEntry()
        {
            IList pos = new ArrayList();
            pos.Add(null);
            source["null"] = pos;
        }

        /// <summary>
        /// Tests set-indexer given negative value.
        /// </summary>
        [Test]
        public void TestIndexerSetNegative()
        {
            // create a new position list
            IList pos = new ArrayList();
            pos.Add(-12345);
            pos.Add(-67890L);
            source["topcoder"] = pos;
            source.Save();

            IList assert = source["topcoder"];
            Assert.AreEqual(2, assert.Count, "Assert count");
            Assert.AreEqual(-12345, assert[0], "Assert 1st element");
            Assert.AreEqual(-67890, assert[1], "Assert 2nd element");
        }

        /// <summary>
        /// Tests indexer trims the word.
        /// </summary>
        [Test]
        public void TestIndexerTrim()
        {
            IList pos = new ArrayList();
            pos.Add(11);
            source[" Trim Test  "] = pos;
            source.Save();

            IList assert = source["Trim Test"];
            Assert.AreEqual(1, assert.Count, "Spaces are trimmed");
            Assert.AreEqual(11, assert[0], "Pos");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Trim Test'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word trimmed");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(11, reader.GetInt64(0), "First loc");
                Assert.IsFalse(reader.Read(), "No row");
            }
        }

        /// <summary>
        /// Tests indexer given an IList with duplicate positions.
        /// Only 1 position is stored.
        /// </summary>
        [Test]
        public void TestIndexerDuplicate()
        {
            IList pos = new ArrayList();
            pos.Add(11);
            pos.Add(11);
            source["Dup"] = pos;
            source.Save();

            IList assert = source["Dup"];
            Assert.AreEqual(1, assert.Count, "Internally it's now 1");
            Assert.AreEqual(11, assert[0], "Pos 0");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Dup'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word persisted");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(11, reader.GetInt64(0), "First loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests Add accuracy by adding a new position to an existing word.
        /// </summary>
        [Test]
        public void TestAdd()
        {
            // add a new position for existing word
            source.Add(new WordInfo("Index", 10));
            source.Save();

            IList assert = source["Index"];
            Assert.AreEqual(2, assert.Count, "Adds a new entry");
            Assert.AreEqual(4, assert[0], "Old element");
            Assert.AreEqual(10, assert[1], "Newly added element");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Index'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word exists");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(4, reader.GetInt64(0), "First loc");
                Assert.IsTrue(reader.Read(), "Second row");
                Assert.AreEqual(10, reader.GetInt64(0), "Second loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests Add accuracy by adding a new position to a non-existing word (case-sensitive).
        /// </summary>
        [Test]
        public void TestAddNewWord()
        {
            // add a new word
            source.Add(new WordInfo("index", 10));
            source.Save();

            IList assert = source["index"];
            Assert.AreEqual(1, assert.Count, "Adds a new entry");
            Assert.AreEqual(10, assert[0], "New position");

            Assert.AreEqual(1, source["Index"].Count, "Old element still exists");
            Assert.AreEqual(4, source["Index"][0], "Old element value");

            // assert the db
            // existing word
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Index'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word exists");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(4, reader.GetInt64(0), "First loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }

            // new word
            long newWordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='index'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word added");
            Assert.IsTrue(wordId != newWordId, "Word added");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + newWordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(10, reader.GetInt64(0), "First loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests Add accuracy by adding IList containing duplicate positions.
        /// It won't be added.
        /// </summary>
        [Test]
        public void TestAddDuplicate()
        {
            // add a duplicate position for existing word
            source.Add(new WordInfo("Index", 4));
            source.Save();

            IList assert = source["Index"];
            Assert.AreEqual(1, assert.Count, "Dup are not added");
            Assert.AreEqual(4, assert[0], "Old element");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Index'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word exists");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(4, reader.GetInt64(0), "First loc");
                Assert.IsFalse(reader.Read(), "No more row");
            }
        }

        /// <summary>
        /// Tests Add trim the word.
        /// </summary>
        [Test]
        public void TestAddTrim()
        {
            source.Add(new WordInfo(" Trim Test  ", 11));
            source.Save();

            IList assert = source["Trim Test"];
            Assert.AreEqual(1, assert.Count, "Spaces are trimmed");
            Assert.AreEqual(11, assert[0], "Pos");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='Trim Test'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word trimmed");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsTrue(reader.Read(), "First row");
                Assert.AreEqual(11, reader.GetInt64(0), "First loc");
                Assert.IsFalse(reader.Read(), "No row");
            }
        }

        /// <summary>
        /// Tests Add given null word.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestAddNullName()
        {
            source.Add(null);
        }

        /// <summary>
        /// Tests Add given empty word.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestAddEmpty()
        {
            source.Add(new WordInfo(" ", 1));
        }

        /// <summary>
        /// Tests Remove accuracy by removing existing name.
        /// </summary>
        [Test]
        public void TestRemove()
        {
            Assert.IsNotNull(source["TopCoder"], "TopCoder before removed");
            source.Remove("TopCoder");
            source.Save();
            Assert.IsNull(source["TopCoder"], "TopCoder is removed");

            // assert the db
            long wordId = Convert.ToInt64(
                new SqlStatement("SELECT Word_Id FROM Word WHERE Word='TopCoder'")
                .ExecuteScalar(conn, null));
            Assert.IsTrue(wordId > 0, "Word still exists");

            using (IDataReader reader = new SqlStatement(
                "SELECT Location FROM Location WHERE Document_Id = " + sampleDocumentId
                + " AND Word_Id = " + wordId
                + " ORDER BY Location")
                .ExecuteReader(conn, null))
            {
                Assert.IsFalse(reader.Read(), "No row in Location");
            }
        }

        /// <summary>
        /// Tests Remove by removing non-existing name (name is case-sensitive).
        /// Nothing should be removed.
        /// </summary>
        [Test]
        public void TestRemoveNotExist()
        {
            Assert.IsNotNull(source["TopCoder"], "TopCoder exists");
            Assert.IsNotNull(source["Open"], "Open exists");
            source.Remove("topcoder");
            source.Save();
            Assert.IsNotNull(source["TopCoder"], "TopCoder still exists");
            Assert.IsNotNull(source["Open"], "Open still exists");

            // reload everything and assert all still exists
            source.Load(param);

            // 2005
            IList pos = source["2005"];
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            pos = source["Document"];
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            pos = source["Index"];
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            pos = source["Open"];
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            pos = source["Persistence"];
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            pos = source["TopCoder"];
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");

            Assert.IsNull(source["NotExists"], "Word not exists");
        }

        /// <summary>
        /// Tests Remove given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestRemoveNull()
        {
            source.Remove(null);
        }

        /// <summary>
        /// Tests Remove given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestRemoveEmpty()
        {
            source.Remove(" ");
        }

        /// <summary>
        /// Tests GetEnumerator accuracy by iterating existing collection.
        /// </summary>
        [Test]
        public void TestGetEnumerator()
        {
            IDictionaryEnumerator it = source.GetEnumerator();

            // 2005
            Assert.IsTrue(it.MoveNext(), "1st entry");
            IList pos = (IList) it.Value;
            Assert.AreEqual("2005", it.Key, "2005");
            Assert.AreEqual(1, pos.Count, "2005 count");
            Assert.AreEqual(2, pos[0], "2005 pos");

            // Document
            Assert.IsTrue(it.MoveNext(), "2nd entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Document", it.Key, "Document");
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            Assert.IsTrue(it.MoveNext(), "3rd entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Index", it.Key, "Index");
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            Assert.IsTrue(it.MoveNext(), "4th entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Open", it.Key, "Open");
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            Assert.IsTrue(it.MoveNext(), "5th entry");
            pos = (IList) it.Value;
            Assert.AreEqual("Persistence", it.Key, "Persistence");
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            Assert.IsTrue(it.MoveNext(), "6th entry");
            pos = (IList) it.Value;
            Assert.AreEqual("TopCoder", it.Key, "TopCoder");
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");

            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Add and GetEnumerator accuracy that a new element is added.
        /// </summary>
        [Test]
        public void TestGetEnumeratorAdd()
        {
            source.Add(new WordInfo("new Word", 10));
            source.Save();

            // reload data
            source.Load(param);
            IDictionaryEnumerator it = source.GetEnumerator();

            // we have 7 entries now
            for (int i = 0; i < 7; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Remove and GetEnumerator accuracy that an element is removed.
        /// </summary>
        [Test]
        public void TestGetEnumeratorRemove()
        {
            source.Remove("Index");
            source.Save();

            // reload data
            source.Load(param);
            IDictionaryEnumerator it = source.GetEnumerator();

            // we have 5 entries now
            for (int i = 0; i < 5; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Entry: " + i);
            }
            // no more entry
            Assert.IsFalse(it.MoveNext(), "No more entry");
        }

        /// <summary>
        /// Tests Contains accuracy by checking existing and non-existing name.
        /// </summary>
        [Test]
        public void TestContains()
        {
            Assert.IsTrue(source.Contains("2005"), "1st entry");
            Assert.IsTrue(source.Contains("Document"), "2nd entry");
            Assert.IsTrue(source.Contains("Index"), "3rd entry");
            Assert.IsTrue(source.Contains("Open"), "4th entry");
            Assert.IsTrue(source.Contains("Persistence"), "5th entry");
            Assert.IsTrue(source.Contains("TopCoder"), "6th entry");
            Assert.IsFalse(source.Contains("document"), "Name is case-sensitive");
        }

        /// <summary>
        /// Tests Contains given null index name.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestContainsNull()
        {
            source.Contains(null);
        }

        /// <summary>
        /// Tests Contains given an empty string index name.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestContainsEmpty()
        {
            source.Contains(" ");
        }

        /// <summary>
        /// Tests Contains given a word with empty position.
        /// False is expected.
        /// </summary>
        [Test]
        public void TestContainsEmptyPosition()
        {
            source["NoPos"] = new ArrayList();
            source.Save();
            Assert.IsFalse(source.Contains("NoPos"), "The array is empty");
        }

        /// <summary>
        /// Tests Clear removes all words for this document.
        /// Assert that it is cleared via Contains, indexer, and GetEnumerator.
        /// </summary>
        [Test]
        public void TestClear()
        {
            source.Clear();
            source.Save();
            Assert.IsFalse(source.Contains("2005"), "1st entry");
            Assert.IsFalse(source.Contains("Document"), "2nd entry");
            Assert.IsFalse(source.Contains("Index"), "3rd entry");
            Assert.IsFalse(source.Contains("Open"), "4th entry");
            Assert.IsFalse(source.Contains("Persistence"), "5th entry");
            Assert.IsFalse(source.Contains("TopCoder"), "6th entry");

            Assert.IsFalse(source.GetEnumerator().MoveNext(), "No entry in enumerator");

            // assert the Location table
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Location"))
            {
                Assert.AreEqual(0, countSql.ExecuteScalar(conn, null), "Location cleared");
            }
        }

        /// <summary>
        /// Tests Compare for default Culture and CompareOptions.
        /// </summary>
        [Test]
        public void TestCompare()
        {
            Assert.AreEqual(0, source.Compare("AC", "AC"), "Equals");
            Assert.AreEqual(1, source.Compare("AC", "ac"), "Lowercase appears first");
            Assert.AreEqual(-1, source.Compare("13", "AC"), "Digits appears first");
        }

        /// <summary>
        /// Tests Compare given non-string first argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareNonString1()
        {
            source.Compare(10, "10");
        }

        /// <summary>
        /// Tests Compare given non-string second argument.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestCompareNonString2()
        {
            source.Compare("10", 10);
        }

        /// <summary>
        /// Tests Compare with invariant culture.
        /// </summary>
        [Test]
        public void TestCompareInvariantCulture()
        {
            source.Clear();
            source.Culture = CultureInfo.InvariantCulture;
            Assert.AreEqual(0, source.Compare("AC", "AC"), "Equals");
            Assert.AreEqual(1, source.Compare("AC", "ac"), "Lowercase appears first");
            Assert.AreEqual(-1, source.Compare("13", "AC"), "Digits appears first");
        }

        /// <summary>
        /// Tests Compare with Ordinal CompareOptions.
        /// </summary>
        [Test]
        public void TestCompareOrdinal()
        {
            source.Clear();
            source.CompareOptions = CompareOptions.Ordinal;
            Assert.AreEqual(0, source.Compare("AC", "AC"), "Equals");
            Assert.IsTrue(source.Compare("AC", "ac") < 0, "Lowercase appears last");
            Assert.IsTrue(source.Compare("13", "AC") < 0, "Digits appears first");
        }

        /// <summary>
        /// Tests Compare with case-insensitive CompareOptions.
        /// </summary>
        [Test]
        public void TestCompareCaseInsensitive()
        {
            source.Clear();
            source.CompareOptions = CompareOptions.IgnoreCase;
            Assert.AreEqual(0, source.Compare("AC", "ac"), "Equals");
            Assert.AreEqual(-1, source.Compare("AC", "af"), "AC appears first (case-insensitive)");
            Assert.AreEqual(-1, source.Compare("13", "AC"), "Digits appears first");
        }

        /// <summary>
        /// Tests case-insensitive CompareOptions with Add/Remove/Contains and indexer.
        /// </summary>
        [Test]
        public void TestCaseInsensitive()
        {
            source.Clear();
            source.CompareOptions = CompareOptions.IgnoreCase;

            // ABC: 10
            IList pos0 = new ArrayList();
            pos0.Add(10);
            source["ABC"] = pos0;

            // abcd: 20
            IList pos1 = new ArrayList();
            pos1.Add(20);
            source["abcd"] = pos1;

            // aBc: 30 (this should overwrite ABC)
            IList pos2 = new ArrayList();
            pos2.Add(30);
            source["aBc"] = pos2;

            // test with get-indexer
            Assert.AreEqual(30, source["abc"][0], "Indexer getter abc is case-insensitive");
            Assert.AreEqual(20, source["AbCd"][0], "Indexer getter abcd is case-insensitive");

            // test with Add (2 positions registered now)
            source.Add(new WordInfo("AbC", 40));
            Assert.AreEqual(30, source["abc"][0], "Add0 should be case-insensitive");
            Assert.AreEqual(40, source["abc"][1], "Add1 should be case-insensitive");

            // test with Remove and Contains
            source.Remove("abc");
            Assert.IsNull(source["abc"], "Remove should be case-insensitive");
            Assert.IsFalse(source.Contains("aBC"), "Contains should be case-insensitive");

            // test Enumertor
            IDictionaryEnumerator it = source.GetEnumerator();
            Assert.IsTrue(it.MoveNext(), "abcd retrieved");
            Assert.AreEqual("abcd", it.Key, "abcd key");

            Assert.IsFalse(it.MoveNext(), "No more element");
        }

        /// <summary>
        /// Tests Load accuracy by inserting some data and load it back.
        /// </summary>
        [Test]
        public void TestLoad()
        {
            // insert 2 words
            source.Add(new WordInfo("2005", 12));
            IList pos = new ArrayList();
            pos.Add(10);
            source["New word"] = pos;
            source.Save();

            // re-load the data from db
            source.Load(param);

            // 7 words should be loaded
            IDictionaryEnumerator it = source.GetEnumerator();
            for (int i = 0; i < 7; i++)
            {
                Assert.IsTrue(it.MoveNext(), "Data is loaded: " + i);
            }
            Assert.IsFalse(it.MoveNext(), "No more data");

            // 2005
            pos = source["2005"];
            Assert.AreEqual(2, pos.Count, "2005 count");
            Assert.AreEqual(2, Math.Min((long) pos[0], (long) pos[1]), "2005 pos0");
            Assert.AreEqual(12, Math.Max((long) pos[0], (long) pos[1]), "2005 pos1");

            // Document
            pos = source["Document"];
            Assert.AreEqual(1, pos.Count, "Document count");
            Assert.AreEqual(3, pos[0], "Document pos");

            // Index
            pos = source["Index"];
            Assert.AreEqual(1, pos.Count, "Index count");
            Assert.AreEqual(4, pos[0], "Index pos");

            // Open
            pos = source["Open"];
            Assert.AreEqual(2, pos.Count, "Open count");
            Assert.AreEqual(1, pos[0], "Open 1 pos");
            Assert.AreEqual(9, pos[1], "Open 2 pos");

            // Persistence
            pos = source["Persistence"];
            Assert.AreEqual(1, pos.Count, "Persistence count");
            Assert.AreEqual(5, pos[0], "Persistence pos");

            // TopCoder
            pos = source["TopCoder"];
            Assert.AreEqual(4, pos.Count, "TopCoder count");
            Assert.AreEqual(0, pos[0], "TopCoder 1 pos");
            Assert.AreEqual(6, pos[1], "TopCoder 2 pos");
            Assert.AreEqual(7, pos[2], "TopCoder 3 pos");
            Assert.AreEqual(8, pos[3], "TopCoder 4 pos");

            // New word
            pos = source["New word"];
            Assert.AreEqual(1, pos.Count, "New word count");
            Assert.AreEqual(10, pos[0], "New word pos");
        }

        /// <summary>
        /// Tests Load roll back any pending uncommitted transaction.
        /// </summary>
        [Test]
        public void TestLoadRollback()
        {
            // insert 2 records (but don't save)
            source.Add(new WordInfo("Open", 33));
            IList pos = new ArrayList();
            pos.Add(111);
            source["New wordx"] = pos;

            // reload (this will clear everything)
            source.Load(param);
            Assert.AreEqual(2, source["Open"].Count, "Added word rolled back");
            Assert.IsNull(source["New wordx"], "New word rolled back");
        }

        /// <summary>
        /// Tests Load given null argument.
        /// Expecting ArgumentNullException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentNullException))]
        public void TestLoadNull()
        {
            source.Load(null);
        }

        /// <summary>
        /// Tests Load given Parameter which is not DBDocumentIndexParameters.
        /// Expecting ArgumentException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(ArgumentException))]
        public void TestLoadNotDBDocumentIndexParameters()
        {
            // give it a DBPersistenceParameters
            source.Load(new DBPersistenceParameters(DbConnectionName));
        }

        /// <summary>
        /// Tests Load given Parameter that points to an invalid connection name.
        /// Expecting DBPersistenceConfigurationException.
        /// </summary>
        [Test]
        [ExpectedException(typeof(DBPersistenceConfigurationException))]
        public void TestLoadInvalid()
        {
            source.Load(new DBDocumentIndexParameters("NotExists", sampleDocumentId));
        }

        /// <summary>
        /// Tests Save given a no transaction, DBCollectionIndexSource.
        /// Nothing will happen.
        /// </summary>
        [Test]
        public void TestSaveNothing()
        {
            // save (this should do nothing)
            source.Save();

            // assert 1 Document
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Document"))
            {
                Assert.AreEqual(1, countSql.ExecuteScalar(conn, null));
            }
            // assert 6 unique words
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Word"))
            {
                Assert.AreEqual(6, countSql.ExecuteScalar(conn, null));
            }
            // assert 10 locations
            using (SqlStatement countSql = new SqlStatement("SELECT COUNT(*) FROM Location"))
            {
                Assert.AreEqual(10, countSql.ExecuteScalar(conn, null));
            }
        }

        /// <summary>
        /// Tests Save given collection with characters that must be escaped (in the xml),
        /// but not necessarily a problem for the database.
        /// </summary>
        [Test]
        public void TestSaveEscapedChars()
        {
            // string with quote, double quote, <, >
            source.Add(new WordInfo("<Test'quote' \"Double quote\"/>", 111));
            source.Save();

            // reloading the file should not raise exception
            source.Load(param);

            IList pos = source["<Test'quote' \"Double quote\"/>"];
            Assert.AreEqual(1, pos.Count, "1 position");
            Assert.AreEqual(111, pos[0], "Position");
        }
    }
}

