/*
 * Demo.cs
 *
 * Document Indexer Persistence
 * Version: 1.0
 *
 * Copyright (c) 2006, TopCoder, Inc. All rights reserved
 *
 * Author: TCSDEVELOPER
 */

using System;
using System.Collections;
using System.Data;
using System.Globalization;
using System.IO;
using NUnit.Framework;
using TopCoder.Data.ConnectionFactory;
using TopCoder.Document.Index.Persistence.DB;
using TopCoder.Document.Index.Persistence.Xml;
using TopCoder.Document.Index.WordStreams;

namespace TopCoder.Document.Index.Persistence.Tests
{
    /// <summary>
    /// A demonstration of component functionality.
    /// </summary>
    [TestFixture]
    public class Demo : TestCommon
    {
        /// <summary>
        /// The collection index xml file for demo.
        /// </summary>
        private const string colFile = @"..\..\test_files\Collection.xml";

        /// <summary>
        /// The original (unmodified) collection index xml file.
        /// </summary>
        private const string colOrigFile = @"..\..\test_files\Collection.orig.xml";

        /// <summary>
        /// The document index xml file for demo.
        /// </summary>
        private const string docFile = @"..\..\test_files\Document0.xml";

        /// <summary>
        /// The original (unmodified) document index xml file.
        /// </summary>
        private const string docOrigFile = @"..\..\test_files\Document0.orig.xml";

        /// <summary>
        /// Temporary subfiles names.
        /// </summary>
        private const string subFiles = @"..\..\test_files\MultipleSub.Test (EraseMe).xml";

        /// <summary>
        /// Sql to deletes all rows from the document/word/location table.
        /// </summary>
        private SqlStatement[] clearsData = new SqlStatement[]
            {
                new SqlStatement("DELETE FROM Location"),
                new SqlStatement("DELETE FROM Word"),
                new SqlStatement("DELETE FROM Document")
            };

        /// <summary>
        /// Loads configuration file and cleans-up environment.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            // Connection Factory and DBDocumentIndexSource configs
            base.LoadConfigFile(ConnectionFactoryConfig,
                @"..\..\test_files\configs\DBDocumentIndexSource.xml");
            CleanUpEnvironment();
        }

        /// <summary>
        /// Cleans up environment and unload config.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            CleanUpEnvironment();
            base.LoadConfigFile();
        }

        /// <summary>
        /// Cleans up by replacing the collection file with the unmodified one
        /// and deleting all rows from the database.
        /// </summary>
        public void CleanUpEnvironment()
        {
            // restore file
            File.Copy(colOrigFile, colFile, true);
            File.Copy(docOrigFile, docFile, true);

            // erase any subfiles created
            string[] subs = Directory.GetFileSystemEntries(@"..\..\test_files", "*EraseMe*");
            foreach (string sub in subs)
            {
                File.Delete(sub);
            }

            // deletes all db records
            DeletesAllData();
        }

        /// <summary>
        /// Deletes all rows from the database.
        /// </summary>
        private void DeletesAllData()
        {
            IDbConnection conn = ConnectionManager.Instance.CreateDbConnection(
                DbConnectionName, new Hashtable());
            try
            {
                conn.Open();
                foreach (SqlStatement stmt in clearsData)
                {
                    stmt.ExecuteNonQuery(conn, null);
                }
            }
            finally
            {
                conn.Close();
            }
        }

        /// <summary>
        /// Demonstrates Xml collection persistence.
        /// </summary>
        [Test]
        public void DemoXmlCollection()
        {
            // load the collection from an xml file
            ICollectionIndexSource xmlSource = new XmlCollectionIndexSource();
            IParameters parameters = new SimpleIndexParameters(colFile);
            xmlSource.Load(parameters);

            // enumerate through indices
            IDictionaryEnumerator enumerator = xmlSource.GetEnumerator();
            int count = 0;
            while (enumerator.MoveNext())
            {
                count++;
            }
            Assert.AreEqual(3, count, "3 document exist");

            // add a new index
            IParameters documentParams = new SimpleIndexParameters(docFile);
            IndexInfo info = new IndexInfo(typeof(XmlDocumentIndexSource).Assembly.GetName().Name,
                typeof(XmlDocumentIndexSource).FullName, documentParams);
            xmlSource.Add("New document", info);

            // add using indexer
            xmlSource["New document"] = info;

            // save the changes
            xmlSource.Save();

            // check index exists
            Assert.IsTrue(xmlSource.Contains("New document"), "New document was added");

            // remove the index
            xmlSource.Remove("New document");
            Assert.IsFalse(xmlSource.Contains("New document"), "New document was removed");

            // use indexer
            IndexInfo retrieved = xmlSource["Document 2"];
            Assert.AreEqual(@"..\..\test_files\Document2.xml",
                ((SimpleIndexParameters) retrieved.Parameters).FileName, "FileName is incorrect");

            // clear all indices
            xmlSource.Clear();
            Assert.IsFalse(xmlSource.GetEnumerator().MoveNext(), "All data cleared");

            // assert collection file path
            Assert.AreEqual(colFile, ((XmlCollectionIndexSource) xmlSource).FilePath, "Xml FilePath");

            // save the changes
            xmlSource.Save();
        }

        /// <summary>
        /// Demonstrates Xml document persistence.
        /// </summary>
        [Test]
        public void DemoXmlDocument()
        {
            // load document words' indices
            IDocumentIndexSource source = new XmlDocumentIndexSource();
            IParameters documentParams = new SimpleIndexParameters(docFile);
            source.Load(documentParams);

            // check CompareOptions and Culture
            Assert.AreEqual(CompareOptions.None, source.CompareOptions, "CompareOptions is None");
            Assert.AreEqual("en-US", source.Culture.Name, "Culture is en-US");

            // check IndexInfo
            IndexInfo idxInfo = source.IndexInfo;
            Assert.AreEqual(docFile, ((SimpleIndexParameters) idxInfo.Parameters).FileName,
                "The document file path is incorrect");

            // enumerate the word indices
            IDictionaryEnumerator enumerator = source.GetEnumerator();
            int count = 0;
            while (enumerator.MoveNext())
            {
                count++;
            }
            Assert.AreEqual(6, count, "6 unique words exist");

            // add a new word
            WordInfo word = new WordInfo("Cat", 102);
            source.Add(word);

            // add using indexer
            IList pos = new ArrayList();
            pos.Add(103);
            pos.Add(105);
            source["Dog"] = pos;

            // save changes
            source.Save();

            // check word exists
            Assert.IsTrue(source.Contains("Cat"), "Cat was added");

            // remove a word
            source.Remove("Cat");
            Assert.IsFalse(source.Contains("Cat"), "Cat was removed");

            // use indexer
            IList getPos = source["Dog"];
            Assert.AreEqual(2, getPos.Count, "2 positions for Dog");

            // set a new file path
            ((XmlDocumentIndexSource) source).FilePath = subFiles;

            // save to multiple files
            ((XmlDocumentIndexSource) source).NumberOfSubFiles = 1;
            source.Save();

            // clear all words
            source.Clear();
            Assert.IsFalse(source.GetEnumerator().MoveNext(), "All data cleared");

            // save the empty file
            source.Save();
        }

        /// <summary>
        /// Demonstrates DB collection persistence.
        /// </summary>
        [Test]
        public void DemoDBCollection()
        {
            ICollectionIndexSource dbSource = null;
            try
            {
                // load the collection from the database
                dbSource = new DBCollectionIndexSource();
                IParameters parameters = new DBPersistenceParameters(DbConnectionName);
                dbSource.Load(parameters);

                // enumerate through indices
                IDictionaryEnumerator enumerator = dbSource.GetEnumerator();
                int count = 0;
                while (enumerator.MoveNext())
                {
                    count++;
                }
                Assert.AreEqual(0, count, "No document exist");

                // add a new index
                IParameters documentParams = new DBDocumentIndexParameters(DbConnectionName, 1);
                IndexInfo info = new IndexInfo(typeof(DBDocumentIndexSource).Assembly.GetName().Name,
                    typeof(DBDocumentIndexSource).FullName, documentParams);
                dbSource.Add("New document", info);

                // add using indexer
                dbSource["New document"] = info;

                // save the changes
                dbSource.Save();

                // check index exists
                Assert.IsTrue(dbSource.Contains("New document"), "New document was added");

                // remove the index
                dbSource.Remove("New document");
                Assert.IsFalse(dbSource.Contains("New document"), "New document was removed");

                // use indexer
                IndexInfo retrieved = dbSource["Document 2"];
                Assert.IsNull(retrieved, "No such document");

                // clear all indices
                dbSource.Clear();
                Assert.IsFalse(dbSource.GetEnumerator().MoveNext(), "All data cleared");

                // save the changes
                dbSource.Save();
            }
            finally
            {
                if (dbSource != null)
                {
                    // dispose db resources
                    ((IDisposable) dbSource).Dispose();
                }
            }
        }

        /// <summary>
        /// Demonstrates DB document persistence.
        /// </summary>
        [Test]
        public void DemoDBDocument()
        {
            IDocumentIndexSource source = null;
            try
            {
                // create a new document collection first
                ICollectionIndexSource dbColl = new DBCollectionIndexSource();
                IParameters colParams = new DBPersistenceParameters(DbConnectionName);
                dbColl.Load(colParams);

                // save and re-load to get the document params
                IParameters documentParams = new DBDocumentIndexParameters(DbConnectionName, 1);
                IndexInfo info = new IndexInfo(typeof(DBDocumentIndexSource).Assembly.GetName().Name,
                    typeof(DBDocumentIndexSource).FullName, documentParams);
                dbColl.Add("New document", info);
                dbColl.Save();
                dbColl.Load(colParams);
                documentParams = dbColl["New document"].Parameters;

                // load document words' indices (initially empty)
                source = new DBDocumentIndexSource();
                source.Load(documentParams);

                // check CompareOptions and Culture
                Assert.AreEqual(CompareOptions.None, source.CompareOptions, "CompareOptions is None");
                Assert.AreEqual("en-US", source.Culture.Name, "Culture is en-US");

                // check IndexInfo
                IndexInfo idxInfo = source.IndexInfo;
                Assert.AreEqual(documentParams, idxInfo.Parameters, "The Parameter is incorrect");

                // enumerate the word indices
                IDictionaryEnumerator enumerator = source.GetEnumerator();
                int count = 0;
                while (enumerator.MoveNext())
                {
                    count++;
                }
                Assert.AreEqual(0, count, "No words exist");

                // add a new word
                WordInfo word = new WordInfo("Cat", 102);
                source.Add(word);

                // add using indexer
                IList pos = new ArrayList();
                pos.Add(103);
                pos.Add(105);
                source["Dog"] = pos;

                // save changes
                source.Save();

                // check word exists
                Assert.IsTrue(source.Contains("Cat"), "Cat was added");

                // remove a word
                source.Remove("Cat");
                Assert.IsFalse(source.Contains("Cat"), "Cat was removed");

                // use indexer
                IList getPos = source["Dog"];
                Assert.AreEqual(2, getPos.Count, "2 positions for Dog");

                // clear all words
                source.Clear();
                Assert.IsFalse(source.GetEnumerator().MoveNext(), "All data cleared");

                // save transaction
                source.Save();
            }
            finally
            {
                if (source != null)
                {
                    // dispose db resources
                    ((IDisposable) source).Dispose();
                }
            }
        }
    }
}

